from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Activity_Delay_test(CoupledDEVS):
	def __init__(self, name = "Activity_Delay_test", gen1_period = 3, gentime_period = 5):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen1"] = self.addSubModel(Generate_Random(name = "gen1", period = gen1_period))
		self.submodels["gentime"] = self.addSubModel(Generate_Random(name = "gentime", period = gentime_period))
		self.submodels["delay"] = self.addSubModel(Activity_Delay(name = "delay"))
		self.connectPorts(self.submodels["gen1"].my_ports["generated"], self.submodels["delay"].my_ports["input"])
		self.connectPorts(self.submodels["gentime"].my_ports["generated"], self.submodels["delay"].my_ports["delay"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen1":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gentime":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "delay":
					return val
		return imm[0]

class Activity_DelayState:
	def __init__(self, name="", delay_time=None, buffer=None):
		self.name = name
		self.delay_time = delay_time
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.delay_time = " + str(self.delay_time) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Activity_Delay(AtomicDEVS):
	def __init__(self, name = "Activity_Delay", delay_time=1.0, buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Activity_DelayState(name="receiving", delay_time=delay_time, buffer=buffer)

		self.my_ports = {"input": self.addInPort("input"), "release": self.addOutPort("release"), "delay": self.addInPort("delay")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "hold_to_release":
			return self.state.delay_time
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "hold_to_release":
				return {"release" : self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_hold_to_release_to_receiving():
			return True
			
		def action_int_hold_to_release_to_receiving():
			return {"buffer" : self.state.buffer, "delay_time" : self.state.delay_time}
			
		if self.state.name == "hold_to_release" and cond_int_hold_to_release_to_receiving():
			return Activity_DelayState(name="receiving", **action_int_hold_to_release_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_hold_to_release():
			if 'input' in inputs.keys() and 'delay' in inputs.keys():
				return False
			if 'input' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_hold_to_release():
			return {"buffer" : inputs['input'], "delay_time" : self.state.delay_time}
			
		def cond_ext_receiving_to_receiving():
			if 'delay' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_receiving():
			return {"buffer" : self.state.buffer, "delay_time" : inputs['delay']}
			
		def cond_ext_hold_to_release_to_hold_to_release():
			if 'delay' in inputs.keys():
				return True
			return False
			
		def action_ext_hold_to_release_to_hold_to_release():
			return {"buffer" : self.state.buffer, "delay_time" : inputs['delay']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_hold_to_release():
			return Activity_DelayState(name="hold_to_release", **action_ext_receiving_to_hold_to_release())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return Activity_DelayState(name="receiving", **action_ext_receiving_to_receiving())


		if self.state.name == "hold_to_release" and cond_ext_hold_to_release_to_hold_to_release():
			return Activity_DelayState(name="hold_to_release", **action_ext_hold_to_release_to_hold_to_release())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, name = "Generate_Random", period=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=period)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

