from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Combine_Test(CoupledDEVS):
	def __init__(self, name = "Combine_Test", gen2_generator_value = 2):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen1"] = self.addSubModel(Generator(name = "gen1"))
		self.submodels["gen2"] = self.addSubModel(Generator(name = "gen2", generator_value = gen2_generator_value))
		self.submodels["com"] = self.addSubModel(Combine(name = "com"))
		self.submodels["sq"] = self.addSubModel(Square(name = "sq"))
		self.connectPorts(self.submodels["gen1"].my_ports["generated"], self.submodels["com"].my_ports["incoming1"])
		self.connectPorts(self.submodels["gen2"].my_ports["generated"], self.submodels["com"].my_ports["incoming2"])
		self.connectPorts(self.submodels["com"].my_ports["outgoing"], self.submodels["sq"].my_ports["input_value"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "sq":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen1":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen2":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "com":
					return val
		return imm[0]

class Combine(CoupledDEVS):
	def __init__(self, name = "Combine"):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"incoming1": self.addInPort("incoming1"), "incoming2": self.addInPort("incoming2"), "outgoing": self.addOutPort("outgoing")}

		self.submodels["pas1"] = self.addSubModel(Passer(name = "pas1"))
		self.submodels["pas2"] = self.addSubModel(Passer(name = "pas2"))
		self.connectPorts(self.my_ports["incoming1"], self.submodels["pas1"].my_ports["incoming"])
		self.connectPorts(self.my_ports["incoming2"], self.submodels["pas2"].my_ports["incoming"])
		self.connectPorts(self.submodels["pas1"].my_ports["outgoing"], self.my_ports["outgoing"])
		self.connectPorts(self.submodels["pas2"].my_ports["outgoing"], self.my_ports["outgoing"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "test":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "pas1":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "pas2":
					return val
		return imm[0]

class PasserState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Passer(AtomicDEVS):
	def __init__(self, name = "Passer", buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = PasserState(name="main", buffer=buffer)

		self.my_ports = {"incoming": self.addInPort("incoming"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "ret":
			return 0
			
		if self.state.name == "main":
			return INFINITY
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "ret":
				return {'outgoing' : self.state.buffer}
				
			if self.state.name == "main":
				return {}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_ret_to_main():
			return True
			
		def action_int_ret_to_main():
			return {'buffer' : self.state.buffer}
			
		if self.state.name == "ret" and cond_int_ret_to_main():
			return PasserState(name="main", **action_int_ret_to_main())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_main_to_ret():
			return 'incoming' in inputs.keys()
			
		def action_ext_main_to_ret():
			return {"buffer" : inputs['incoming']}
			
		if self.state.name == "main" and cond_ext_main_to_ret():
			return PasserState(name="ret", **action_ext_main_to_ret())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

class SquareState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Square(AtomicDEVS):
	def __init__(self, name = "Square", buffer=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = SquareState(name="receiving", buffer=buffer)

		self.my_ports = {"input_value": self.addInPort("input_value"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "calculating_square":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "calculating_square":
				return {"result" : self.state.buffer ** 2}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_calculating_square_to_receiving():
			return True
			
		def action_int_calculating_square_to_receiving():
			return {"buffer" : 1}
			
		if self.state.name == "calculating_square" and cond_int_calculating_square_to_receiving():
			return SquareState(name="receiving", **action_int_calculating_square_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_calculating_square():
			return isinstance(inputs['input_value'], (int, long, float))
			
		def action_ext_receiving_to_calculating_square():
			return {"buffer" : inputs['input_value']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_calculating_square():
			return SquareState(name="calculating_square", **action_ext_receiving_to_calculating_square())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


