from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class CombineState:
	def __init__(self, name="", buffer2=None, buffer1=None):
		self.name = name
		self.buffer2 = buffer2
		self.buffer1 = buffer1


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer2 = " + str(self.buffer2) + "\n"
		s += "state.buffer1 = " + str(self.buffer1) + "\n"
		return s

class Combine(AtomicDEVS):
	def __init__(self, name = "Combine", buffer2=None, buffer1=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = CombineState(name="incoming", buffer2=buffer2, buffer1=buffer1)

		self.my_ports = {"incoming1": self.addInPort("incoming1"), "incoming2": self.addInPort("incoming2"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "incoming":
			return INFINITY
			
		if self.state.name == "outgoing":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "incoming":
				return {}
				
			if self.state.name == "outgoing":
				return {"outgoing" : buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_outgoing_to_incoming():
			return True
			
		def action_int_outgoing_to_incoming():
			return {"buffer1" : self.state.buffer1, "buffer2" : self.state.buffer2}
			
		if self.state.name == "outgoing" and cond_int_outgoing_to_incoming():
			return CombineState(name="incoming", **action_int_outgoing_to_incoming())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_incoming_to_outgoing():
			if 'incoming1' in inputs.keys():
				return True
			if 'incoming2' in inputs.keys():
				return True
			return False
			
		def action_ext_incoming_to_outgoing():
			if 'incoming1' in inputs.keys() and 'incoming2' in input.keys():
			if 'incoming1' in inputs.keys():
				return {"buffer1" : inputs['incoming1'], "buffer2" : self.state.buffer2}
			if 'incoming2' in inputs.keys():
				return {"buffer2" : inputs['incoming2'], "buffer1" : self.state.buffer1}
			
		if self.state.name == "incoming" and cond_ext_incoming_to_outgoing():
			return CombineState(name="outgoing", **action_ext_incoming_to_outgoing())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Combine(CoupledDEVS):
	def __init__(self, name = "Combine"):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"incoming1": self.addInPort("incoming1"), "incoming2": self.addInPort("incoming2"), "outgoing": self.addOutPort("outgoing")}

		self.submodels["pas1"] = self.addSubModel(Passer(name = "pas1"))
		self.submodels["pas2"] = self.addSubModel(Passer(name = "pas2"))
		self.connectPorts(self.my_ports["incoming1"], self.submodels["pas1"].my_ports["incoming"])
		self.connectPorts(self.my_ports["incoming2"], self.submodels["pas2"].my_ports["incoming"])
		self.connectPorts(self.submodels["pas1"].my_ports["outgoing"], self.my_ports["outgoing"])
		self.connectPorts(self.submodels["pas2"].my_ports["outgoing"], self.my_ports["outgoing"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "test":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "pas1":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "pas2":
					return val
		return imm[0]

class PasserState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Passer(AtomicDEVS):
	def __init__(self, name = "Passer", buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = PasserState(name="main", buffer=buffer)

		self.my_ports = {"incoming": self.addInPort("incoming"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "ret":
			return 0
			
		if self.state.name == "main":
			return INFINITY
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "ret":
				return {'outgoing' : self.state.buffer}
				
			if self.state.name == "main":
				return {}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_ret_to_main():
			return True
			
		def action_int_ret_to_main():
			return {'buffer' : self.state.buffer}
			
		if self.state.name == "ret" and cond_int_ret_to_main():
			return PasserState(name="main", **action_int_ret_to_main())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_main_to_ret():
			return 'incoming' in inputs.keys()
			
		def action_ext_main_to_ret():
			return {"buffer" : inputs['incoming']}
			
		if self.state.name == "main" and cond_ext_main_to_ret():
			return PasserState(name="ret", **action_ext_main_to_ret())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


