from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Count_Items_test(CoupledDEVS):
	def __init__(self, name = "Count_Items_test", get_generator_period = 2, reset_generator_period = 10):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generator(name = "gen"))
		self.submodels["get"] = self.addSubModel(Generator(name = "get", generator_period = get_generator_period))
		self.submodels["reset"] = self.addSubModel(Generator(name = "reset", generator_period = reset_generator_period))
		self.submodels["count"] = self.addSubModel(Count_Items(name = "count"))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["count"].my_ports["incoming"])
		self.connectPorts(self.submodels["get"].my_ports["generated"], self.submodels["count"].my_ports["get"])
		self.connectPorts(self.submodels["reset"].my_ports["generated"], self.submodels["count"].my_ports["reset"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "count":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "get":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "reset":
					return val
		return imm[0]

class Count_ItemsState:
	def __init__(self, name="", gotten=None, send_on_receive=None, buffer=None, count=None):
		self.name = name
		self.gotten = gotten
		self.send_on_receive = send_on_receive
		self.buffer = buffer
		self.count = count


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.gotten = " + str(self.gotten) + "\n"
		s += "state.send_on_receive = " + str(self.send_on_receive) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		s += "state.count = " + str(self.count) + "\n"
		return s

class Count_Items(AtomicDEVS):
	def __init__(self, name = "Count_Items", gotten=False, send_on_receive=False, buffer=None, count=0):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Count_ItemsState(name="receiving", gotten=gotten, send_on_receive=send_on_receive, buffer=buffer, count=count)

		self.my_ports = {"incoming": self.addInPort("incoming"), "get": self.addInPort("get"), "reset": self.addInPort("reset"), "outgoing": self.addOutPort("outgoing"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				if self.state.gotten or self.state.send_on_receive:
					self.state.gotten = False
					if self.state.buffer == None:
						return {"result" : self.state.count}
					else:
						return {"outgoing" : self.state.buffer, "result" : self.state.count}
				else:
					if self.state.buffer == None:
						return {}
					else:
						return {"outgoing" : self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {"buffer" : self.state.buffer, "count" : self.state.count, "send_on_receive" : self.state.send_on_receive, "gotten" : self.state.gotten}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return Count_ItemsState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_receiving():
			if 'reset' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_receiving():
			return {"buffer" : self.state.buffer, "count" : 0, "send_on_receive" : self.state.send_on_receive, "gotten" : self.state.gotten}
			
		def cond_ext_receiving_to_sending():
			self.state.gotten = 'get' in inputs.keys()
			if 'get' in inputs.keys() or 'incoming' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_sending():
			if 'incoming' in inputs.keys():
				self.state.count = self.state.count + 1
				self.state.buffer = inputs['incoming']
			else:
				self.state.buffer = None
			return {"buffer" : self.state.buffer, "count" : self.state.count, "send_on_receive" : self.state.send_on_receive, "gotten" : self.state.gotten}
			
		def cond_ext_sending_to_sending():
			if 'reset' in inputs.keys():
				self.state.count = 0
			self.state.gotten = 'get' in inputs.keys()
			if 'get' in inputs.keys() or 'incoming' in inputs.keys():
				return True
			return False
			
		def action_ext_sending_to_sending():
			if 'incoming' in inputs.keys():
				self.state.count = self.state.count + 1
				self.state.buffer = inputs['incoming']
			else:
				self.state.buffer = None
			return {"buffer" : self.state.buffer, "count" : self.state.count, "send_on_receive" : self.state.send_on_receive, "gotten" : self.state.gotten}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return Count_ItemsState(name="receiving", **action_ext_receiving_to_receiving())


		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return Count_ItemsState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "sending" and cond_ext_sending_to_sending():
			return Count_ItemsState(name="sending", **action_ext_sending_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

