from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Queue_FIFO_test(CoupledDEVS):
	def __init__(self, name = "Queue_FIFO_test", fifo_max_size = 5, rel_generator_period = 1, rel_generator_value = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["fifo"] = self.addSubModel(Queue_FIFO(name = "fifo", max_size = fifo_max_size))
		self.submodels["gen"] = self.addSubModel(Generate_Random(name = "gen"))
		self.submodels["rel"] = self.addSubModel(Generator(name = "rel", generator_period = rel_generator_period, generator_value = rel_generator_value))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["fifo"].my_ports["add"])
		self.connectPorts(self.submodels["rel"].my_ports["generated"], self.submodels["fifo"].my_ports["get"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "fifo":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "rel":
					return val
		return imm[0]

class Queue_FIFOState:
	def __init__(self, name="", max_size=None, buffer=None):
		self.name = name
		self.max_size = max_size
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.max_size = " + str(self.max_size) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Queue_FIFO(AtomicDEVS):
	def __init__(self, name = "Queue_FIFO", max_size=INFINITY, buffer=[]):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Queue_FIFOState(name="receiving", max_size=max_size, buffer=buffer)

		self.my_ports = {"add": self.addInPort("add"), "get": self.addInPort("get"), "release": self.addOutPort("release"), "av_queue_length": self.addOutPort("av_queue_length"), "av_wait_time": self.addOutPort("av_wait_time"), "utilization": self.addOutPort("utilization"), "request_stat": self.addInPort("request_stat")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "data":
			return 0
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "data":
				return {}
				
			if self.state.name == "sending":
				if len(self.state.buffer) != 0:
					result = self.state.buffer.pop()
					return {'release' : result}
				return {}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {"buffer" : self.state.buffer, "max_size" : self.state.max_size}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return Queue_FIFOState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		print(inputs)
		def cond_ext_receiving_to_receiving():
			if 'add' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_receiving():
			if len(self.state.buffer) < self.state.max_size:
				self.state.buffer.insert(0, inputs['add'])
			return {"buffer" : self.state.buffer, "max_size" : self.state.max_size}
			
		def cond_ext_receiving_to_sending():
			if 'get' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_sending():
			return {"buffer" : self.state.buffer, "max_size" : self.state.max_size}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return Queue_FIFOState(name="receiving", **action_ext_receiving_to_receiving())


		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return Queue_FIFOState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, name = "Generate_Random", period=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=period)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

