from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class message:
	def __init__(self, points=None):
		self.points = points


	def __str__(self):
		s = ""
		s += "state.points = " + str(self.points) + "\n"
		return s


class QueuePriorityTest(CoupledDEVS):
	def __init__(self, name = "QueuePriorityTest", queue_size = 5, queue_override = True, gen_generator_value = message(points = 5), queue_priority_name = 'points'):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["random"] = self.addSubModel(Generate_Random(name = "random"))
		self.submodels["queue"] = self.addSubModel(Queue_Priority(name = "queue", size = queue_size, override = queue_override, priority_name = queue_priority_name))
		self.submodels["gen"] = self.addSubModel(Generator(name = "gen", generator_value = gen_generator_value))

		def transfer_gen_generated_to_queue_input(event):
			import random
			#event['points'] = int(random.random()*100)
			event.points = int(random.random()*100)
			return event

		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["queue"].my_ports["input"], transfer_gen_generated_to_queue_input)

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "random":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "queue":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		return imm[0]

class Queue_PriorityState:
	def __init__(self, name="", override=None, size=None, priority_name=None, queue_priorities=None, queue=None):
		self.name = name
		self.override = override
		self.size = size
		self.priority_name = priority_name
		self.queue_priorities = queue_priorities
		self.queue = queue


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.override = " + str(self.override) + "\n"
		s += "state.size = " + str(self.size) + "\n"
		s += "state.priority_name = " + str(self.priority_name) + "\n"
		s += "state.queue_priorities = " + str(self.queue_priorities) + "\n"
		s += "state.queue = " + str(self.queue) + "\n"
		return s

class Queue_Priority(AtomicDEVS):
	def __init__(self, name = "Queue_Priority", override=False, size=INFINITY, priority_name='priority', queue_priorities=[], queue=[]):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Queue_PriorityState(name="receiving", override=override, size=size, priority_name=priority_name, queue_priorities=queue_priorities, queue=queue)

		self.my_ports = {"input": self.addInPort("input"), "get": self.addInPort("get"), "output": self.addOutPort("output")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				ret = self.state.queue.pop()
				self.state.queue_priorities.pop()
				return {'output' : ret}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'queue' : self.state.queue, 'priority_name' : self.state.priority_name, 'queue_priorities' : self.state.queue_priorities, 'size' : self.state.size, 'override' : self.state.override}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return Queue_PriorityState(name="receiving", **action_int_sending_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'get' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_sending():
			return {'queue' : self.state.queue, 'priority_name' : self.state.priority_name, 'queue_priorities' : self.state.queue_priorities, 'size' : self.state.size, 'override' : self.state.override}
			
		def cond_ext_receiving_to_receiving():
			if 'input' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_receiving():
			adder = inputs['input']
			priority = 1
			if isinstance(adder, (int, long, float)):
				priority = adder
			else:
				try:
					priority = getattr(adder, self.state.priority_name)
				except Exception:
					try:
						priority = adder.get(self.state.priority_name, None)
					except Exception:
						priority = 1
			if not isinstance(priority, (int, long, float)):
				priority = 1
			if len(self.state.queue_priorities) < self.state.size or (self.state.override == True and priority > self.state.queue_priorities[0]):
				if len(self.state.queue_priorities) >= self.state.size:
					self.state.queue.pop(0)
					self.state.queue_priorities.pop(0)
				if len(self.state.queue_priorities) == 0:
					self.state.queue.append(adder)
					self.state.queue_priorities.append(priority)
				else:
					i = 0
					while i < len(self.state.queue_priorities) and self.state.queue_priorities[i] < priority:
						i = i + 1
					self.state.queue.insert(i, adder)
					self.state.queue_priorities.insert(i, priority)
			else:
				print("Queue is full, incoming item will be dropped")
			return {'queue' : self.state.queue, 'priority_name' : self.state.priority_name, 'queue_priorities' : self.state.queue_priorities, 'size' : self.state.size, 'override' : self.state.override}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return Queue_PriorityState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return Queue_PriorityState(name="receiving", **action_ext_receiving_to_receiving())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, name = "Generate_Random", period=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=period)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

