from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Timer_test(CoupledDEVS):
	def __init__(self, name = "Timer_test", del_delay_time = 1.66):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generator(name = "gen"))
		self.submodels["del"] = self.addSubModel(Activity_Delay(name = "del", delay_time = del_delay_time))
		self.submodels["timer"] = self.addSubModel(Timer(name = "timer"))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["del"].my_ports["input"])
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["timer"].my_ports["start"])
		self.connectPorts(self.submodels["del"].my_ports["release"], self.submodels["timer"].my_ports["stop"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "del":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "timer":
					return val
		return imm[0]

class TimerState:
	def __init__(self, name="", end_time=None, start_time=None):
		self.name = name
		self.end_time = end_time
		self.start_time = start_time


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.end_time = " + str(self.end_time) + "\n"
		s += "state.start_time = " + str(self.start_time) + "\n"
		return s

class Timer(AtomicDEVS):
	def __init__(self, name = "Timer", end_time=0.0, start_time=0.0):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = TimerState(name="standby", end_time=end_time, start_time=start_time)

		self.my_ports = {"start": self.addInPort("start"), "stop": self.addInPort("stop"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "running":
			return INFINITY
			
		if self.state.name == "standby":
			return INFINITY
			
		if self.state.name == "finished":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "running":
				return {}
				
			if self.state.name == "standby":
				return {}
				
			if self.state.name == "finished":
				return {'result' : self.state.end_time - self.state.start_time}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_finished_to_standby():
			return True
			
		def action_int_finished_to_standby():
			return {"start_time" : self.state.start_time, "end_time" : self.state.end_time}
			
		if self.state.name == "finished" and cond_int_finished_to_standby():
			return TimerState(name="standby", **action_int_finished_to_standby())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_standby_to_running():
			return 'start' in inputs.keys()
			
		def action_ext_standby_to_running():
			self.elapsed = 0
			return {"start_time" : self.elapsed, "end_time" : self.elapsed}
			
		def cond_ext_running_to_finished():
			self.state.end_time = self.state.end_time + self.elapsed
			return 'stop' in inputs.keys()
			
		def action_ext_running_to_finished():
			return {"start_time" : self.state.start_time, "end_time" : self.state.end_time}
			
		if self.state.name == "standby" and cond_ext_standby_to_running():
			return TimerState(name="running", **action_ext_standby_to_running())


		if self.state.name == "running" and cond_ext_running_to_finished():
			return TimerState(name="finished", **action_ext_running_to_finished())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Activity_DelayState:
	def __init__(self, name="", delay_time=None, buffer=None):
		self.name = name
		self.delay_time = delay_time
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.delay_time = " + str(self.delay_time) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Activity_Delay(AtomicDEVS):
	def __init__(self, name = "Activity_Delay", delay_time=1.0, buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Activity_DelayState(name="receiving", delay_time=delay_time, buffer=buffer)

		self.my_ports = {"input": self.addInPort("input"), "release": self.addOutPort("release"), "delay": self.addInPort("delay")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "hold_to_release":
			return self.state.delay_time - self.elapsed
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "hold_to_release":
				return {"release" : self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_hold_to_release_to_receiving():
			return True
			
		def action_int_hold_to_release_to_receiving():
			return {"buffer" : self.state.buffer, "delay_time" : self.state.delay_time}
			
		if self.state.name == "hold_to_release" and cond_int_hold_to_release_to_receiving():
			return Activity_DelayState(name="receiving", **action_int_hold_to_release_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_receiving():
			if 'delay' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_receiving():
			return {"buffer" : self.state.buffer, "delay_time" : inputs['delay']}
			
		def cond_ext_hold_to_release_to_hold_to_release():
			if 'delay' in inputs.keys():
				return True
			return False
			
		def action_ext_hold_to_release_to_hold_to_release():
			return {"buffer" : self.state.buffer, "delay_time" : inputs['delay']}
			
		def cond_ext_receiving_to_hold_to_release():
			if 'input' in inputs.keys() and 'delay' in inputs.keys():
				return False
			if 'input' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_hold_to_release():
			self.elapsed = 0
			return {"buffer" : inputs['input'], "delay_time" : self.state.delay_time}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return Activity_DelayState(name="receiving", **action_ext_receiving_to_receiving())


		if self.state.name == "hold_to_release" and cond_ext_hold_to_release_to_hold_to_release():
			return Activity_DelayState(name="hold_to_release", **action_ext_hold_to_release_to_hold_to_release())


		if self.state.name == "receiving" and cond_ext_receiving_to_hold_to_release():
			return Activity_DelayState(name="hold_to_release", **action_ext_receiving_to_hold_to_release())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

