from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class TimerState:
	def __init__(self, name="", end_time=None, start_time=None):
		self.name = name
		self.end_time = end_time
		self.start_time = start_time


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.end_time = " + str(self.end_time) + "\n"
		s += "state.start_time = " + str(self.start_time) + "\n"
		return s

class Timer(AtomicDEVS):
	def __init__(self, name = "Timer", end_time=0.0, start_time=0.0):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = TimerState(name="standby", end_time=end_time, start_time=start_time)

		self.my_ports = {"start": self.addInPort("start"), "stop": self.addInPort("stop"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "running":
			return INFINITY
			
		if self.state.name == "standby":
			return INFINITY
			
		if self.state.name == "finished":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "running":
				return {}
				
			if self.state.name == "standby":
				return {}
				
			if self.state.name == "finished":
				return {'result' : self.state.end_time - self.state.start_time}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_finished_to_standby():
			return True
			
		def action_int_finished_to_standby():
			return {"start_time" : self.state.start_time, "end_time" : self.state.end_time}
			
		if self.state.name == "finished" and cond_int_finished_to_standby():
			return TimerState(name="standby", **action_int_finished_to_standby())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_standby_to_running():
			return 'start' in inputs.keys()
			
		def action_ext_standby_to_running():
			self.elapsed = 0
			return {"start_time" : self.elapsed, "end_time" : self.elapsed}
			
		def cond_ext_running_to_finished():
			self.state.end_time = self.state.end_time + self.elapsed
			return 'stop' in inputs.keys()
			
		def action_ext_running_to_finished():
			return {"start_time" : self.state.start_time, "end_time" : self.state.end_time}
			
		if self.state.name == "standby" and cond_ext_standby_to_running():
			return TimerState(name="running", **action_ext_standby_to_running())


		if self.state.name == "running" and cond_ext_running_to_finished():
			return TimerState(name="finished", **action_ext_running_to_finished())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


