from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class unbatch_test(CoupledDEVS):
	def __init__(self, name = "unbatch_test", get1_generator_period = 2, get2_generator_period = 3, batch_auto_send = False, batch_incoming1_amount = 2):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["batch"] = self.addSubModel(Batch(name = "batch", auto_send = batch_auto_send, incoming1_amount = batch_incoming1_amount))
		self.submodels["unbatch"] = self.addSubModel(Unbatch(name = "unbatch"))
		self.submodels["get1"] = self.addSubModel(Generator(name = "get1", generator_period = get1_generator_period))
		self.submodels["get2"] = self.addSubModel(Generator(name = "get2", generator_period = get2_generator_period))
		self.submodels["gen"] = self.addSubModel(Generate_Random(name = "gen"))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["batch"].my_ports["incoming1"])
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["batch"].my_ports["incoming2"])
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["batch"].my_ports["incoming3"])
		self.connectPorts(self.submodels["get1"].my_ports["generated"], self.submodels["batch"].my_ports["demand"])
		self.connectPorts(self.submodels["batch"].my_ports["outgoing"], self.submodels["unbatch"].my_ports["incoming"])
		self.connectPorts(self.submodels["get2"].my_ports["generated"], self.submodels["unbatch"].my_ports["demand"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "batch":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "unbatch":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "get1":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "get2":
					return val
		return imm[0]

class UnbatchState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Unbatch(AtomicDEVS):
	def __init__(self, name = "Unbatch", buffer=[]):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = UnbatchState(name="receiving", buffer=buffer)

		self.my_ports = {"incoming": self.addInPort("incoming"), "outgoing1": self.addOutPort("outgoing1"), "outgoing2": self.addOutPort("outgoing2"), "outgoing3": self.addOutPort("outgoing3"), "demand": self.addInPort("demand")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				if len(self.state.buffer) == 0:
					return {}
				buf = self.state.buffer[0]
				sen1 = None
				sen2 = None
				sen3 = None
				ret = {}
				if len(buf[0]) > 0:
					sen1 = buf[0].pop()
					ret['outgoing1'] = sen1
				if len(buf[1]) > 0:
					sen2 = buf[1].pop()
					ret['outgoing2'] = sen2
				if len(buf[2]) > 0:
					sen3 = buf[2].pop()
					ret['outgoing3'] = sen3
				if len(buf[0]) == 0 and len(buf[1]) == 0 and len(buf[2]) == 0:
					self.state.buffer.pop(0)
				return ret
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'buffer' : self.state.buffer}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return UnbatchState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'demand' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_sending():
			return {'buffer' : self.state.buffer}
			
		def cond_ext_receiving_to_receiving():
			if 'incoming' in inputs.keys():
				valu = inputs['incoming']
				try:
					if len(valu) != 3:
						return False
					if not isinstance(valu[0], list):
						return False
					if not isinstance(valu[1], list):
						return False
					if not isinstance(valu[2], list):
						return False
					return True
				except  Exception:
					return False
			return False
			
		def action_ext_receiving_to_receiving():
			self.state.buffer.append(inputs['incoming'])
			return {'buffer' : self.state.buffer}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return UnbatchState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return UnbatchState(name="receiving", **action_ext_receiving_to_receiving())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class BatchState:
	def __init__(self, name="", auto_send=None, buffer3=None, buffer2=None, buffer1=None, incoming3_amount=None, incoming2_amount=None, incoming1_amount=None):
		self.name = name
		self.auto_send = auto_send
		self.buffer3 = buffer3
		self.buffer2 = buffer2
		self.buffer1 = buffer1
		self.incoming3_amount = incoming3_amount
		self.incoming2_amount = incoming2_amount
		self.incoming1_amount = incoming1_amount


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.auto_send = " + str(self.auto_send) + "\n"
		s += "state.buffer3 = " + str(self.buffer3) + "\n"
		s += "state.buffer2 = " + str(self.buffer2) + "\n"
		s += "state.buffer1 = " + str(self.buffer1) + "\n"
		s += "state.incoming3_amount = " + str(self.incoming3_amount) + "\n"
		s += "state.incoming2_amount = " + str(self.incoming2_amount) + "\n"
		s += "state.incoming1_amount = " + str(self.incoming1_amount) + "\n"
		return s

class Batch(AtomicDEVS):
	def __init__(self, name = "Batch", auto_send=True, buffer3=[], buffer2=[], buffer1=[], incoming3_amount=1, incoming2_amount=1, incoming1_amount=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = BatchState(name="receiving", auto_send=auto_send, buffer3=buffer3, buffer2=buffer2, buffer1=buffer1, incoming3_amount=incoming3_amount, incoming2_amount=incoming2_amount, incoming1_amount=incoming1_amount)

		self.my_ports = {"incoming1": self.addInPort("incoming1"), "incoming2": self.addInPort("incoming2"), "incoming3": self.addInPort("incoming3"), "outgoing": self.addOutPort("outgoing"), "demand": self.addInPort("demand")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				result = []
				result1 = []
				result2 = []
				result3 = []
				for x in range(0, self.state.incoming1_amount):
					result1.insert(0, self.state.buffer1.pop())
				for x in range(0, self.state.incoming2_amount):
					result2.insert(0, self.state.buffer2.pop())
				for x in range(0, self.state.incoming3_amount):
					result3.insert(0, self.state.buffer3.pop())
				result.append(result1)
				result.append(result2)
				result.append(result3)
				return {'outgoing' : result}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'incoming1_amount' : self.state.incoming1_amount, 'incoming2_amount' : self.state.incoming2_amount, 'incoming3_amount' : self.state.incoming3_amount, 'buffer1' : self.state.buffer1, 'buffer2' : self.state.buffer2, 'buffer3' : self.state.buffer3, 'auto_send' : self.state.auto_send}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return BatchState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_receiving():
			if 'incoming1' in inputs.keys():
				if len(self.state.buffer1) < (self.state.incoming1_amount - 1) or len(self.state.buffer2) < self.state.incoming2_amount or len(self.state.buffer3) < self.state.incoming3_amount or self.state.auto_send == False:
					return True
			if 'incoming2' in inputs.keys():
				if len(self.state.buffer2) < (self.state.incoming2_amount - 1) or len(self.state.buffer1) < self.state.incoming1_amount or len(self.state.buffer3) < self.state.incoming3_amount or self.state.auto_send == False:
					return True
			if 'incoming3' in inputs.keys():
				if len(self.state.buffer3) < (self.state.incoming3_amount - 1) or len(self.state.buffer2) < self.state.incoming2_amount or len(self.state.buffer1) < self.state.incoming1_amount or self.state.auto_send == False:
					return True
			return False
			
		def action_ext_receiving_to_receiving():
			if 'incoming1' in inputs.keys():
				self.state.buffer1.insert(0, inputs['incoming1'])
			if 'incoming2' in inputs.keys():
				self.state.buffer2.insert(0, inputs['incoming2'])
			if 'incoming3' in inputs.keys():
				self.state.buffer3.insert(0, inputs['incoming3'])
			return {'incoming1_amount' : self.state.incoming1_amount, 'incoming2_amount' : self.state.incoming2_amount, 'incoming3_amount' : self.state.incoming3_amount, 'buffer1' : self.state.buffer1, 'buffer2' : self.state.buffer2, 'buffer3' : self.state.buffer3,  'auto_send' : self.state.auto_send}
			
		def cond_ext_receiving_to_sending():
			if 'incoming1' in inputs.keys():
				if len(self.state.buffer1) == (self.state.incoming1_amount - 1) and len(self.state.buffer2) >= self.state.incoming2_amount and len(self.state.buffer3) >= self.state.incoming3_amount and self.state.auto_send:
					return True
			if 'incoming2' in inputs.keys():
				if len(self.state.buffer2) == (self.state.incoming2_amount - 1) and len(self.state.buffer1) >= self.state.incoming1_amount and len(self.state.buffer3) >= self.state.incoming3_amount and self.state.auto_send:
					return True
			if 'incoming3' in inputs.keys():
				if len(self.state.buffer3) == (self.state.incoming3_amount - 1) and len(self.state.buffer2) >= self.state.incoming2_amount and len(self.state.buffer1) >= self.state.incoming1_amount and self.state.auto_send:
					return True
			if 'demand' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_sending():
			if 'incoming1' in inputs.keys():
				self.state.buffer1.insert(0, inputs['incoming1'])
			if 'incoming2' in inputs.keys():
				self.state.buffer2.insert(0, inputs['incoming2'])
			if 'incoming3' in inputs.keys():
				self.state.buffer3.insert(0, inputs['incoming3'])
			return {'incoming1_amount' : self.state.incoming1_amount, 'incoming2_amount' : self.state.incoming2_amount, 'incoming3_amount' : self.state.incoming3_amount, 'buffer1' : self.state.buffer1, 'buffer2' : self.state.buffer2, 'buffer3' : self.state.buffer3,  'auto_send' : self.state.auto_send}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return BatchState(name="receiving", **action_ext_receiving_to_receiving())


		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return BatchState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, name = "Generate_Random", period=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=period)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

