from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class FIFOState:
	def __init__(self, name="", elapsed_time=None, bound=None, time=None, pop_amount=None, buffer=None):
		self.name = name
		self.elapsed_time = elapsed_time
		self.bound = bound
		self.time = time
		self.pop_amount = pop_amount
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.elapsed_time = " + str(self.elapsed_time) + "\n"
		s += "state.bound = " + str(self.bound) + "\n"
		s += "state.time = " + str(self.time) + "\n"
		s += "state.pop_amount = " + str(self.pop_amount) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class FIFO(AtomicDEVS):
	def __init__(self, parameters={"name" : "FIFO"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = FIFOState(name="receiving", elapsed_time=0, bound=INFINITY, time=INFINITY, pop_amount=1, buffer=[])

		self.my_ports = {"add": self.addInPort("add"), "get": self.addInPort("get"), "output": self.addOutPort("output"), "time": self.addInPort("time"), "bound": self.addInPort("bound")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return self.state.time - self.state.elapsed_time
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				pops = self.state.pop_amount
				if pops < 1:
					print("requested to pop negative or 0  number of values, will return 1")
					pops = 1
				result = []
				if pops > len(self.state.buffer):
					pops = len(self.state.buffer)
				for x in range(0, pops):
					result.insert(0, self.state.buffer.pop())
				return {"output" : result}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : self.state.time, "bound" : self.state.bound, "elapsed_time" : 0}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return FIFOState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'get' in inputs.keys():
				return isinstance(inputs['get'], int)
			else:
				return False
			
		def action_ext_receiving_to_sending():
			return {"buffer" : self.state.buffer, "pop_amount" : inputs['get'], "time" : self.state.time, "bound" : self.state.bound, "elapsed_time" : self.state.elapsed_time}
			
		def cond_ext_receiving_to_receiving():
			if 'add' in inputs.keys():
				return True
			if 'time' in inputs.keys():
				if isinstance(inputs['time'], int) or inputs['time'] == INFINITY:
					return True
			if 'bound' in inputs.keys():
				if isinstance(inputs['bound'], int) or inputs['bound'] == INFINITY:
					return True
			return False
			
		def action_ext_receiving_to_receiving():
			if 'time' in inputs.keys():
				return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : inputs['time'], "bound" : self.state.bound, "elapsed_time" : self.state.elapsed_time}
			if 'bound' in inputs.keys():
				return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : self.state.time, "bound" : inputs['bound'], "elapsed_time" : self.state.elapsed_time}
			if 'add' in inputs.keys():
				self.state.elapsed_time += self.elapsed
				if len(self.state.buffer) < self.state.bound:
					self.state.buffer.insert(0, inputs['add'])
				return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : self.state.time, "bound" : self.state.bound, "elapsed_time" : self.state.elapsed_time}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return FIFOState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return FIFOState(name="receiving", **action_ext_receiving_to_receiving())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


