from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class OR_test(CoupledDEVS):
	def __init__(self, name = "OR_test", gen1_generator_value = 1, gen2_generator_value = 0):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["ore"] = self.addSubModel(OR(name = "ore"))
		self.submodels["gen1"] = self.addSubModel(Generator(name = "gen1", generator_value = gen1_generator_value))
		self.submodels["gen2"] = self.addSubModel(Generator(name = "gen2", generator_value = gen2_generator_value))
		self.submodels["ande"] = self.addSubModel(AND(name = "ande"))
		self.submodels["note"] = self.addSubModel(NOT(name = "note"))
		self.submodels["xore"] = self.addSubModel(XOR(name = "xore"))
		self.connectPorts(self.submodels["gen1"].my_ports["generated"], self.submodels["ore"].my_ports["incoming1"])
		self.connectPorts(self.submodels["gen2"].my_ports["generated"], self.submodels["ore"].my_ports["incoming2"])
		self.connectPorts(self.submodels["gen1"].my_ports["generated"], self.submodels["ande"].my_ports["incoming1"])
		self.connectPorts(self.submodels["gen2"].my_ports["generated"], self.submodels["ande"].my_ports["incoming2"])
		self.connectPorts(self.submodels["gen1"].my_ports["generated"], self.submodels["xore"].my_ports["incoming1"])
		self.connectPorts(self.submodels["gen2"].my_ports["generated"], self.submodels["xore"].my_ports["incoming2"])
		self.connectPorts(self.submodels["gen1"].my_ports["generated"], self.submodels["note"].my_ports["value"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen1":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen2":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "or":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "ore":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "ande":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "note":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "xore":
					return val
		return imm[0]

class ORState:
	def __init__(self, name="", buf2=None, buf1=None):
		self.name = name
		self.buf2 = buf2
		self.buf1 = buf1


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buf2 = " + str(self.buf2) + "\n"
		s += "state.buf1 = " + str(self.buf1) + "\n"
		return s

class OR(AtomicDEVS):
	def __init__(self, name = "OR", buf2=None, buf1=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = ORState(name="zero", buf2=buf2, buf1=buf1)

		self.my_ports = {"incoming1": self.addInPort("incoming1"), "incoming2": self.addInPort("incoming2"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "zero":
			return INFINITY
			
		if self.state.name == "one":
			return INFINITY
			
		if self.state.name == "two":
			return INFINITY
			
		if self.state.name == "send":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "zero":
				return {}
				
			if self.state.name == "one":
				return {}
				
			if self.state.name == "two":
				return {}
				
			if self.state.name == "send":
				if self.state.buf1 > 0 or self.state.buf2 > 0:
					return {'outgoing' : 1}
				return {'outgoing' : 0}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_send_to_zero():
			return True
			
		def action_int_send_to_zero():
			return {'buf1' : self.state.buf1, 'buf2' : self.state.buf2}
			
		if self.state.name == "send" and cond_int_send_to_zero():
			return ORState(name="zero", **action_int_send_to_zero())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_two_to_send():
			if 'incoming1' in inputs.keys():
				if isinstance(inputs['incoming1'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_two_to_send():
			return {'buf1' : inputs['incoming1'], 'buf2' : self.state.buf2}
			
		def cond_ext_zero_to_two():
			if 'incoming2' in inputs.keys():
				if isinstance(inputs['incoming2'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_zero_to_two():
			return {'buf2' : inputs['incoming2'], 'buf1' : self.state.buf1}
			
		def cond_ext_zero_to_one():
			if 'incoming1' in inputs.keys():
				if isinstance(inputs['incoming1'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_zero_to_one():
			return {'buf1' : inputs['incoming1'], 'buf2' : self.state.buf2}
			
		def cond_ext_one_to_send():
			if 'incoming2' in inputs.keys():
				if isinstance(inputs['incoming2'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_one_to_send():
			return {'buf2' : inputs['incoming2'], 'buf1' : self.state.buf1}
			
		if self.state.name == "two" and cond_ext_two_to_send():
			return ORState(name="send", **action_ext_two_to_send())


		if self.state.name == "zero" and cond_ext_zero_to_two():
			return ORState(name="two", **action_ext_zero_to_two())


		if self.state.name == "zero" and cond_ext_zero_to_one():
			return ORState(name="one", **action_ext_zero_to_one())


		if self.state.name == "one" and cond_ext_one_to_send():
			return ORState(name="send", **action_ext_one_to_send())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

class ANDState:
	def __init__(self, name="", buf2=None, buf1=None):
		self.name = name
		self.buf2 = buf2
		self.buf1 = buf1


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buf2 = " + str(self.buf2) + "\n"
		s += "state.buf1 = " + str(self.buf1) + "\n"
		return s

class AND(AtomicDEVS):
	def __init__(self, name = "AND", buf2=None, buf1=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = ANDState(name="zero", buf2=buf2, buf1=buf1)

		self.my_ports = {"incoming1": self.addInPort("incoming1"), "incoming2": self.addInPort("incoming2"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "zero":
			return INFINITY
			
		if self.state.name == "one":
			return INFINITY
			
		if self.state.name == "two":
			return INFINITY
			
		if self.state.name == "send":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "zero":
				return {}
				
			if self.state.name == "one":
				return {}
				
			if self.state.name == "two":
				return {}
				
			if self.state.name == "send":
				if self.state.buf1 > 0 and self.state.buf2 > 0:
					return {'outgoing' : 1}
				return {'outgoing' : 0}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_send_to_zero():
			return True
			
		def action_int_send_to_zero():
			return {'buf1' : self.state.buf1, 'buf2' : self.state.buf2}
			
		if self.state.name == "send" and cond_int_send_to_zero():
			return ANDState(name="zero", **action_int_send_to_zero())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_two_to_send():
			if 'incoming1' in inputs.keys():
				if isinstance(inputs['incoming1'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_two_to_send():
			return {'buf1' : inputs['incoming1'], 'buf2' : self.state.buf2}
			
		def cond_ext_zero_to_two():
			if 'incoming2' in inputs.keys():
				if isinstance(inputs['incoming2'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_zero_to_two():
			return {'buf2' : inputs['incoming2'], 'buf1' : self.state.buf1}
			
		def cond_ext_zero_to_one():
			if 'incoming1' in inputs.keys():
				if isinstance(inputs['incoming1'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_zero_to_one():
			return {'buf1' : inputs['incoming1'], 'buf2' : self.state.buf2}
			
		def cond_ext_one_to_send():
			if 'incoming2' in inputs.keys():
				if isinstance(inputs['incoming2'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_one_to_send():
			return {'buf2' : inputs['incoming2'], 'buf1' : self.state.buf1}
			
		if self.state.name == "two" and cond_ext_two_to_send():
			return ANDState(name="send", **action_ext_two_to_send())


		if self.state.name == "zero" and cond_ext_zero_to_two():
			return ANDState(name="two", **action_ext_zero_to_two())


		if self.state.name == "zero" and cond_ext_zero_to_one():
			return ANDState(name="one", **action_ext_zero_to_one())


		if self.state.name == "one" and cond_ext_one_to_send():
			return ANDState(name="send", **action_ext_one_to_send())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class NOTState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class NOT(AtomicDEVS):
	def __init__(self, name = "NOT", buffer=0.0):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = NOTState(name="receiving", buffer=buffer)

		self.my_ports = {"value": self.addInPort("value"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "received_inverse":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "received_inverse":
				if self.state.buffer > 0:
					return {'result' : 0}
				return {'result' : 1}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_received_inverse_to_receiving():
			return True
			
		def action_int_received_inverse_to_receiving():
			return {"buffer" : self.state.buffer}
			
		if self.state.name == "received_inverse" and cond_int_received_inverse_to_receiving():
			return NOTState(name="receiving", **action_int_received_inverse_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_received_inverse():
			return isinstance(inputs['value'], (int, long, float))
			
		def action_ext_receiving_to_received_inverse():
			return {"buffer" :  inputs['value']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_received_inverse():
			return NOTState(name="received_inverse", **action_ext_receiving_to_received_inverse())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class XORState:
	def __init__(self, name="", buf2=None, buf1=None):
		self.name = name
		self.buf2 = buf2
		self.buf1 = buf1


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buf2 = " + str(self.buf2) + "\n"
		s += "state.buf1 = " + str(self.buf1) + "\n"
		return s

class XOR(AtomicDEVS):
	def __init__(self, name = "XOR", buf2=None, buf1=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = XORState(name="zero", buf2=buf2, buf1=buf1)

		self.my_ports = {"incoming1": self.addInPort("incoming1"), "incoming2": self.addInPort("incoming2"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "zero":
			return INFINITY
			
		if self.state.name == "one":
			return INFINITY
			
		if self.state.name == "two":
			return INFINITY
			
		if self.state.name == "send":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "zero":
				return {}
				
			if self.state.name == "one":
				return {}
				
			if self.state.name == "two":
				return {}
				
			if self.state.name == "send":
				if self.state.buf1 > 0 or self.state.buf2 > 0:
					if self.state.buf1 > 0 and self.state.buf2 > 0:
						return {'outgoing' : 0}
					return {'outgoing' : 1}
				return {'outgoing' : 0}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_send_to_zero():
			return True
			
		def action_int_send_to_zero():
			return {'buf1' : self.state.buf1, 'buf2' : self.state.buf2}
			
		if self.state.name == "send" and cond_int_send_to_zero():
			return XORState(name="zero", **action_int_send_to_zero())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_two_to_send():
			if 'incoming1' in inputs.keys():
				if isinstance(inputs['incoming1'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_two_to_send():
			return {'buf1' : inputs['incoming1'], 'buf2' : self.state.buf2}
			
		def cond_ext_zero_to_two():
			if 'incoming2' in inputs.keys():
				if isinstance(inputs['incoming2'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_zero_to_two():
			return {'buf2' : inputs['incoming2'], 'buf1' : self.state.buf1}
			
		def cond_ext_zero_to_one():
			if 'incoming1' in inputs.keys():
				if isinstance(inputs['incoming1'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_zero_to_one():
			return {'buf1' : inputs['incoming1'], 'buf2' : self.state.buf2}
			
		def cond_ext_one_to_send():
			if 'incoming2' in inputs.keys():
				if isinstance(inputs['incoming2'], (int, long, float, complex)):
					return True
			return False
			
		def action_ext_one_to_send():
			return {'buf2' : inputs['incoming2'], 'buf1' : self.state.buf1}
			
		if self.state.name == "two" and cond_ext_two_to_send():
			return XORState(name="send", **action_ext_two_to_send())


		if self.state.name == "zero" and cond_ext_zero_to_two():
			return XORState(name="two", **action_ext_zero_to_two())


		if self.state.name == "zero" and cond_ext_zero_to_one():
			return XORState(name="one", **action_ext_zero_to_one())


		if self.state.name == "one" and cond_ext_one_to_send():
			return XORState(name="send", **action_ext_one_to_send())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


