from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class circularqueuetest(CoupledDEVS):
	def __init__(self, name = "circularqueuetest", gen_generator_period = 3, queue_override = True):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generator(name = "gen", generator_period = gen_generator_period))
		self.submodels["add"] = self.addSubModel(Generate_Random(name = "add"))
		self.submodels["queue"] = self.addSubModel(CircularQueue(name = "queue", override = queue_override))

		def transfer_add_generated_to_queue_incoming(event):
			return int(event*100)

		self.connectPorts(self.submodels["add"].my_ports["generated"], self.submodels["queue"].my_ports["incoming"], transfer_add_generated_to_queue_incoming)
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["queue"].my_ports["get"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "queue":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "add":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		return imm[0]

class CircularQueueState:
	def __init__(self, name="", queue=None, front=None, rear=None, override=None):
		self.name = name
		self.queue = queue
		self.front = front
		self.rear = rear
		self.override = override


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.queue = " + str(self.queue) + "\n"
		s += "state.front = " + str(self.front) + "\n"
		s += "state.rear = " + str(self.rear) + "\n"
		s += "state.override = " + str(self.override) + "\n"
		return s

class CircularQueue(AtomicDEVS):
	def __init__(self, name = "CircularQueue", queue=[None] * 6, front=-1, rear=-1, override=False):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = CircularQueueState(name="receiving", queue=queue, front=front, rear=rear, override=override)

		self.my_ports = {"incoming": self.addInPort("incoming"), "get": self.addInPort("get"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				size = len(self.state.queue)
				if self.state.front == -1 and self.state.rear == -1:
					return {}
				retval = self.state.queue[self.state.front]
				self.state.front = self.state.front + 1
				if self.state.front == size:
					self.state.front = 0
				if self.state.front - 1 == self.state.rear:
					self.state.front = -1
					self.state.rear = -1
				return {'outgoing' : retval}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'override' : self.state.override, 'rear' : self.state.rear, 'front' : self.state.front, 'queue' : self.state.queue}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return CircularQueueState(name="receiving", **action_int_sending_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'get' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_sending():
			return {'override' : self.state.override, 'rear' : self.state.rear, 'front' : self.state.front, 'queue' : self.state.queue}
			
		def cond_ext_receiving_to_receiving():
			if 'incoming' in inputs.keys():
				return True
			return False
			
		def action_ext_receiving_to_receiving():
			size = len(self.state.queue)
			if self.state.override == False and ((self.state.rear == size-1 and self.state.front == 0) or (self.state.front == self.state.rear + 1)):
				print('queue is full')
				return {'override' : self.state.override, 'rear' : self.state.rear, 'front' : self.state.front, 'queue' : self.state.queue}
			else:
				if self.state.rear == size-1 and self.state.front != 0:
					self.state.rear = -1
			self.state.rear = self.state.rear + 1
			self.state.queue[self.state.rear] = inputs['incoming']
			if self.state.override == True and self.state.front == self.state.rear:
				self.state.front = self.state.front + 1
				if self.state.front == size:
					self.state.front = 0
			if self.state.front == -1:
				self.state.front = 0
			return {'override' : self.state.override, 'rear' : self.state.rear, 'front' : self.state.front, 'queue' : self.state.queue}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return CircularQueueState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return CircularQueueState(name="receiving", **action_ext_receiving_to_receiving())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, name = "Generate_Random", period=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=period)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

