from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class CollectorTest(CoupledDEVS):
	def __init__(self, name = "CollectorTest", g2_period = 5):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generate_Random(name = "gen"))
		self.submodels["col"] = self.addSubModel(Collector(name = "col"))
		self.submodels["g2"] = self.addSubModel(Generate_Random(name = "g2", period = g2_period))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["col"].my_ports["input"])
		self.connectPorts(self.submodels["g2"].my_ports["generated"], self.submodels["col"].my_ports["getCount"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "col":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "g2":
					return val
		return imm[0]

class Collector(CoupledDEVS):
	def __init__(self, name = "Collector"):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"input": self.addInPort("input"), "passthrough": self.addOutPort("passthrough"), "getAll": self.addInPort("getAll"), "getTimes": self.addInPort("getTimes"), "getItems": self.addInPort("getItems"), "getCount": self.addInPort("getCount"), "count": self.addOutPort("count"), "times": self.addOutPort("times"), "items": self.addOutPort("items")}

		self.submodels["sim"] = self.addSubModel(SimulationTime(name = "sim"))
		self.submodels["col"] = self.addSubModel(CollectorAtomic(name = "col"))
		self.connectPorts(self.my_ports["input"], self.submodels["sim"].my_ports["incoming"])
		self.connectPorts(self.my_ports["getAll"], self.submodels["col"].my_ports["getAll"])
		self.connectPorts(self.my_ports["getTimes"], self.submodels["col"].my_ports["getTimes"])
		self.connectPorts(self.my_ports["getItems"], self.submodels["col"].my_ports["getItems"])
		self.connectPorts(self.my_ports["getCount"], self.submodels["col"].my_ports["getCount"])
		self.connectPorts(self.submodels["sim"].my_ports["time"], self.submodels["col"].my_ports["time"])
		self.connectPorts(self.submodels["sim"].my_ports["passthrough"], self.submodels["col"].my_ports["incoming"])
		self.connectPorts(self.submodels["col"].my_ports["passthrough"], self.my_ports["passthrough"])
		self.connectPorts(self.submodels["col"].my_ports["count"], self.my_ports["count"])
		self.connectPorts(self.submodels["col"].my_ports["times"], self.my_ports["times"])
		self.connectPorts(self.submodels["col"].my_ports["items"], self.my_ports["items"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "sim":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "col":
					return val
		return imm[0]

class CollectorAtomicState:
	def __init__(self, name="", arrival_times=None, arrival_items=None, count=None, buffer=None):
		self.name = name
		self.arrival_times = arrival_times
		self.arrival_items = arrival_items
		self.count = count
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.arrival_times = " + str(self.arrival_times) + "\n"
		s += "state.arrival_items = " + str(self.arrival_items) + "\n"
		s += "state.count = " + str(self.count) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class CollectorAtomic(AtomicDEVS):
	def __init__(self, name = "CollectorAtomic", arrival_times=[], arrival_items=[], count=0, buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = CollectorAtomicState(name="receiving", arrival_times=arrival_times, arrival_items=arrival_items, count=count, buffer=buffer)

		self.my_ports = {"incoming": self.addInPort("incoming"), "getAll": self.addInPort("getAll"), "getTimes": self.addInPort("getTimes"), "getItems": self.addInPort("getItems"), "getCount": self.addInPort("getCount"), "passthrough": self.addOutPort("passthrough"), "count": self.addOutPort("count"), "times": self.addOutPort("times"), "items": self.addOutPort("items"), "time": self.addInPort("time")}

	def timeAdvance(self):
		if self.state.name == "all":
			return 0
			
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "times":
			return 0
			
		if self.state.name == "count":
			return 0
			
		if self.state.name == "sending":
			return 0
			
		if self.state.name == "items":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "all":
				return {'count' : self.state.count, 'times' : self.state.arrival_times, 'items' : self.state.arrival_items}
				
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "times":
				return {'times' : self.state.arrival_times}
				
			if self.state.name == "count":
				return {'count' : self.state.count}
				
			if self.state.name == "sending":
				return {'passthrough' : self.state.buffer}
				
			if self.state.name == "items":
				return {'items' : self.state.arrival_items}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_incoming_to_receiving():
			return True
			
		def action_int_incoming_to_receiving():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_int_count_to_receiving():
			return True
			
		def action_int_count_to_receiving():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_int_times_to_receiving():
			return True
			
		def action_int_times_to_receiving():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_int_items_to_receiving():
			return True
			
		def action_int_items_to_receiving():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_int_all_to_receiving():
			return True
			
		def action_int_all_to_receiving():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		if self.state.name == "incoming" and cond_int_incoming_to_receiving():
			return CollectorAtomicState(name="receiving", **action_int_incoming_to_receiving())


		if self.state.name == "count" and cond_int_count_to_receiving():
			return CollectorAtomicState(name="receiving", **action_int_count_to_receiving())


		if self.state.name == "times" and cond_int_times_to_receiving():
			return CollectorAtomicState(name="receiving", **action_int_times_to_receiving())


		if self.state.name == "items" and cond_int_items_to_receiving():
			return CollectorAtomicState(name="receiving", **action_int_items_to_receiving())


		if self.state.name == "all" and cond_int_all_to_receiving():
			return CollectorAtomicState(name="receiving", **action_int_all_to_receiving())


		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return CollectorAtomicState(name="receiving", **action_int_sending_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_all():
			return 'getAll' in inputs.keys()
			
		def action_ext_receiving_to_all():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_ext_receiving_to_count():
			return 'getCount' in inputs.keys()
			
		def action_ext_receiving_to_count():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_ext_receiving_to_times():
			return 'getTimes' in inputs.keys()
			
		def action_ext_receiving_to_times():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_ext_receiving_to_items():
			return 'getItems' in inputs.keys()
			
		def action_ext_receiving_to_items():
			return {'buffer' : self.state.buffer, 'count' : self.state.count, 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		def cond_ext_receiving_to_sending():
			return 'incoming' in inputs.keys() and 'time' in inputs.keys()
			
		def action_ext_receiving_to_sending():
			self.state.arrival_items.append(inputs['incoming'])
			self.state.arrival_times.append(inputs['time'])
			return {'buffer' : inputs['incoming'], 'count' : self.state.count +1 , 'arrival_items' : self.state.arrival_items, 'arrival_times' : self.state.arrival_times}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_all():
			return CollectorAtomicState(name="all", **action_ext_receiving_to_all())


		if self.state.name == "receiving" and cond_ext_receiving_to_count():
			return CollectorAtomicState(name="count", **action_ext_receiving_to_count())


		if self.state.name == "receiving" and cond_ext_receiving_to_times():
			return CollectorAtomicState(name="times", **action_ext_receiving_to_times())


		if self.state.name == "receiving" and cond_ext_receiving_to_items():
			return CollectorAtomicState(name="items", **action_ext_receiving_to_items())


		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return CollectorAtomicState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class SimulationTimeState:
	def __init__(self, name="", buffer=None, time=None):
		self.name = name
		self.buffer = buffer
		self.time = time


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		s += "state.time = " + str(self.time) + "\n"
		return s

class SimulationTime(AtomicDEVS):
	def __init__(self, name = "SimulationTime", buffer=None, time=0.0):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = SimulationTimeState(name="receiving", buffer=buffer, time=time)

		self.my_ports = {"incoming": self.addInPort("incoming"), "get": self.addInPort("get"), "passthrough": self.addOutPort("passthrough"), "time": self.addOutPort("time")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "gotten":
			return 0
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "gotten":
				return {'time' : self.state.time}
				
			if self.state.name == "sending":
				return {'passthrough' : self.state.buffer, 'time' : self.state.time}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			self.state.time += self.timeAdvance()
			return {'time' : self.state.time, 'buffer' : self.state.buffer}
			
		def cond_int_gotten_to_receiving():
			return True
			
		def action_int_gotten_to_receiving():
			self.state.time += self.timeAdvance()
			return {'time' : self.state.time, 'buffer' : self.state.buffer}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return SimulationTimeState(name="receiving", **action_int_sending_to_receiving())


		if self.state.name == "gotten" and cond_int_gotten_to_receiving():
			return SimulationTimeState(name="receiving", **action_int_gotten_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_gotten():
			return 'get' in inputs.keys()
			
		def action_ext_receiving_to_gotten():
			return {'time' : self.state.time + self.elapsed, 'buffer' : None}
			
		def cond_ext_receiving_to_sending():
			return 'incoming' in inputs.keys()
			
		def action_ext_receiving_to_sending():
			return {'time' : self.state.time + self.elapsed, 'buffer' : inputs['incoming']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_gotten():
			return SimulationTimeState(name="gotten", **action_ext_receiving_to_gotten())


		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return SimulationTimeState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, name = "Generate_Random", period=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=period)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

