from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class ListLizeTest(CoupledDEVS):
	def __init__(self, name = "ListLizeTest", gen_generator_value = {'test' : 'a', 'b' : 'b', 'c' : 'c', 'd' : 'd'}):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generator(name = "gen", generator_value = gen_generator_value))
		self.submodels["size"] = self.addSubModel(ListSize(name = "size"))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["size"].my_ports["incoming"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "size":
					return val
		return imm[0]

class ListSizeState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class ListSize(AtomicDEVS):
	def __init__(self, name = "ListSize", buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = ListSizeState(name="receiving", buffer=buffer)

		self.my_ports = {"incoming": self.addInPort("incoming"), "passthrough": self.addOutPort("passthrough"), "size": self.addOutPort("size")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				try:
					return {'passthrough' : self.state.buffer, 'size' : len(self.state.buffer)}
				except Exception:
					return {'passthrough': self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'buffer' : self.state.buffer}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return ListSizeState(name="receiving", **action_int_sending_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			return 'incoming' in inputs.keys()
			
		def action_ext_receiving_to_sending():
			return {'buffer' : inputs['incoming']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return ListSizeState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, value=None):
		self.name = name
		self.period = period
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, name = "atomic_generator", period=1, value=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=period, value=value)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, name = "Generator", generator_value = 1, generator_period = 1):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(name = "generator", value = generator_value, period = generator_period))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "generator":
					return val
		return imm[0]

