from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class SimulationTimeState:
	def __init__(self, name="", buffer=None, time=None):
		self.name = name
		self.buffer = buffer
		self.time = time


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		s += "state.time = " + str(self.time) + "\n"
		return s

class SimulationTime(AtomicDEVS):
	def __init__(self, name = "SimulationTime", buffer=None, time=0.0):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = SimulationTimeState(name="receiving", buffer=buffer, time=time)

		self.my_ports = {"incoming": self.addInPort("incoming"), "get": self.addInPort("get"), "passthrough": self.addOutPort("passthrough"), "time": self.addOutPort("time")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "gotten":
			return 0
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "gotten":
				return {'time' : self.state.time}
				
			if self.state.name == "sending":
				return {'passthrough' : self.state.buffer, 'time' : self.state.time}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			self.state.time += self.timeAdvance()
			return {'time' : self.state.time, 'buffer' : self.state.buffer}
			
		def cond_int_gotten_to_receiving():
			return True
			
		def action_int_gotten_to_receiving():
			self.state.time += self.timeAdvance()
			return {'time' : self.state.time, 'buffer' : self.state.buffer}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return SimulationTimeState(name="receiving", **action_int_sending_to_receiving())


		if self.state.name == "gotten" and cond_int_gotten_to_receiving():
			return SimulationTimeState(name="receiving", **action_int_gotten_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_gotten():
			return 'get' in inputs.keys()
			
		def action_ext_receiving_to_gotten():
			return {'time' : self.state.time + self.elapsed, 'buffer' : None}
			
		def cond_ext_receiving_to_sending():
			return 'incoming' in inputs.keys()
			
		def action_ext_receiving_to_sending():
			return {'time' : self.state.time + self.elapsed, 'buffer' : inputs['incoming']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_gotten():
			return SimulationTimeState(name="gotten", **action_ext_receiving_to_gotten())


		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return SimulationTimeState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


