from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Multiplier(CoupledDEVS):
	def __init__(self, parameters={"name" : "Multiplier"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"mult_one": self.addInPort("mult_one"), "mult_two": self.addInPort("mult_two"), "result": self.addOutPort("result")}

		parameters.update({"name" : "multiplier"})
		self.submodels["multiplier"] = self.addSubModel(Atomic_multiplier(parameters=parameters))
		self.connectPorts(self.my_ports["mult_one"], self.submodels["multiplier"].my_ports["mult_one"])
		self.connectPorts(self.my_ports["mult_two"], self.submodels["multiplier"].my_ports["mult_two"])
		self.connectPorts(self.submodels["multiplier"].my_ports["result"], self.my_ports["result"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, Atomic_multiplier):
				return val
		return imm[0]

class Atomic_multiplierState:
	def __init__(self, name="", buffer_two=None, buffer_one=None):
		self.name = name
		self.buffer_two = buffer_two
		self.buffer_one = buffer_one


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer_two = " + str(self.buffer_two) + "\n"
		s += "state.buffer_one = " + str(self.buffer_one) + "\n"
		return s

class Atomic_multiplier(AtomicDEVS):
	def __init__(self, parameters={"name" : "Atomic_multiplier"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = Atomic_multiplierState(name="zero_received", buffer_two=0.0, buffer_one=0.0)

		self.my_ports = {"mult_one": self.addInPort("mult_one"), "mult_two": self.addInPort("mult_two"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "zero_received":
			return INFINITY
			
		if self.state.name == "two_received":
			return 0
			
		if self.state.name == "one_received":
			return INFINITY
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "zero_received":
				return {}
				
			if self.state.name == "two_received":
				return {"result" : self.state.buffer_one * self.state.buffer_two}
				
			if self.state.name == "one_received":
				return {}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_two_received_to_zero_received():
			return True
			
		def action_int_two_received_to_zero_received():
			return {"buffer_one": 0.0, "buffer_two": 0.0}
			
		if self.state.name == "two_received" and cond_int_two_received_to_zero_received():
			return Atomic_multiplierState(name="zero_received", **action_int_two_received_to_zero_received())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_zero_received_to_one_received():
			if 'mult_one' in inputs.keys() and 'mult_two' in inputs.keys():
				return isinstance(inputs['mult_one'], (int, long, float, complex)) and isinstance(inputs['mult_two'], (int, long, float, complex))
			if 'mult_one' in inputs.keys():
				return isinstance(inputs['mult_one'], (int, long, float, complex))
			if 'mult_two' in inputs.keys():
				return isinstance(inputs['mult_two'], (int, long, float, complex))
			
		def action_ext_zero_received_to_one_received():
			if 'mult_one' in inputs.keys() and 'mult_two' in inputs.keys():
				if isinstance(inputs['mult_one'], (int, long, float, complex)) and isinstance(inputs['mult_two'], (int, long, float, complex)):
					return {"buffer_one" : inputs['mult_one'], "buffer_two" : inputs['mult_two']}
			if 'mult_one' in inputs.keys():
				if isinstance(inputs['mult_one'], (int, long, float, complex)):
					return {"buffer_one" : inputs['mult_one'], "buffer_two" : self.state.buffer_two}
			if 'mult_two' in inputs.keys():
				if isinstance(inputs['mult_two'], (int, long, float, complex)):
					return {"buffer_one" : self.state.buffer_one, "buffer_two" : inputs['mult_two']}
			
		def cond_ext_one_received_to_two_received():
			if 'mult_one' in inputs.keys() and 'mult_two' in inputs.keys():
				return isinstance(inputs['mult_one'], (int, long, float, complex)) and isinstance(inputs['mult_two'], (int, long, float, complex))
			if 'mult_one' in inputs.keys():
				return isinstance(inputs['mult_one'], (int, long, float, complex))
			if 'mult_two' in inputs.keys():
				return isinstance(inputs['mult_two'], (int, long, float, complex))
			
		def action_ext_one_received_to_two_received():
			if 'mult_one' in inputs.keys() and 'mult_two' in inputs.keys():
				if isinstance(inputs['mult_one'], (int, long, float, complex)) and isinstance(inputs['mult_two'], (int, long, float, complex)):
					return {"buffer_one" : inputs['mult_one'], "buffer_two" : inputs['mult_two']}
			if 'mult_one' in inputs.keys():
				if isinstance(inputs['mult_one'], (int, long, float, complex)):
					return {"buffer_one" : inputs['mult_one'], "buffer_two" : self.state.buffer_two}
			if 'mult_two' in inputs.keys():
				if isinstance(inputs['mult_two'], (int, long, float, complex)):
					return {"buffer_one" : self.state.buffer_one, "buffer_two" : inputs['mult_two']}
			
		if self.state.name == "zero_received" and cond_ext_zero_received_to_one_received():
			return Atomic_multiplierState(name="one_received", **action_ext_zero_received_to_one_received())


		if self.state.name == "one_received" and cond_ext_one_received_to_two_received():
			return Atomic_multiplierState(name="two_received", **action_ext_one_received_to_two_received())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


