from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class fifo_test(CoupledDEVS):
	def __init__(self, parameters={"name" : "fifo_test"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["fifo"] = self.addSubModel(FIFO(parameters={"name" : "fifo"}))
		self.submodels["time"] = self.addSubModel(Generator_single_instant(parameters={"name" : "time", "value" : parameters["time_value"]}))
		self.submodels["bound"] = self.addSubModel(Generator_single_instant(parameters={"name" : "bound", "value" : parameters["bound_value"]}))
		self.submodels["inputs"] = self.addSubModel(Generator(parameters={"name" : "inputs", "generate_value" : parameters["inputs_generate_value"], "generate_period" : parameters["inputs_generate_period"]}))
		self.submodels["test"] = self.addSubModel(Generator_single_instant(parameters={"name" : "test", "value" : parameters["test_value"]}))
		self.submodels["get"] = self.addSubModel(Generator(parameters={"name" : "get", "generate_value" : parameters["get_generate_value"], "generate_period" : parameters["get_generate_period"]}))
		self.connectPorts(self.submodels["time"].my_ports["generated"], self.submodels["fifo"].my_ports["time"])
		self.connectPorts(self.submodels["bound"].my_ports["generated"], self.submodels["fifo"].my_ports["bound"])
		self.connectPorts(self.submodels["test"].my_ports["generated"], self.submodels["fifo"].my_ports["add"])
		self.connectPorts(self.submodels["get"].my_ports["generated"], self.submodels["fifo"].my_ports["get"])
		self.connectPorts(self.submodels["inputs"].my_ports["generated"], self.submodels["fifo"].my_ports["add"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "fifo":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "time":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "bound":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "inputs":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "test":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "get":
					return val
		return imm[0]

class Generator_single_instantState:
	def __init__(self, name="", hold=None, value=None):
		self.name = name
		self.hold = hold
		self.value = value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.hold = " + str(self.hold) + "\n"
		s += "state.value = " + str(self.value) + "\n"
		return s

class Generator_single_instant(AtomicDEVS):
	def __init__(self, parameters={"name" : "Generator_single_instant"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = Generator_single_instantState(name="generating", hold=0, value=parameters["value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.hold
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"value" : self.state.value, "hold" : INFINITY}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generator_single_instantState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class FIFOState:
	def __init__(self, name="", elapsed_time=None, bound=None, time=None, pop_amount=None, buffer=None):
		self.name = name
		self.elapsed_time = elapsed_time
		self.bound = bound
		self.time = time
		self.pop_amount = pop_amount
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.elapsed_time = " + str(self.elapsed_time) + "\n"
		s += "state.bound = " + str(self.bound) + "\n"
		s += "state.time = " + str(self.time) + "\n"
		s += "state.pop_amount = " + str(self.pop_amount) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class FIFO(AtomicDEVS):
	def __init__(self, parameters={"name" : "FIFO"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = FIFOState(name="receiving", elapsed_time=0, bound=INFINITY, time=INFINITY, pop_amount=1, buffer=[])

		self.my_ports = {"add": self.addInPort("add"), "get": self.addInPort("get"), "output": self.addOutPort("output"), "time": self.addInPort("time"), "bound": self.addInPort("bound")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return self.state.time - self.state.elapsed_time
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				pops = self.state.pop_amount
				if pops < 1:
					print("requested to pop negative or 0  number of values, will return 1")
					pops = 1
				result = []
				if pops > len(self.state.buffer):
					pops = len(self.state.buffer)
				for x in range(0, pops):
					result.insert(0, self.state.buffer.pop())
				return {"output" : result}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : self.state.time, "bound" : self.state.bound, "elapsed_time" : 0}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return FIFOState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'get' in inputs.keys():
				return isinstance(inputs['get'], int)
			else:
				return False
			
		def action_ext_receiving_to_sending():
			return {"buffer" : self.state.buffer, "pop_amount" : inputs['get'], "time" : self.state.time, "bound" : self.state.bound, "elapsed_time" : self.state.elapsed_time}
			
		def cond_ext_receiving_to_receiving():
			if 'add' in inputs.keys():
				return True
			if 'time' in inputs.keys():
				if isinstance(inputs['time'], int) or inputs['time'] == INFINITY:
					return True
			if 'bound' in inputs.keys():
				if isinstance(inputs['bound'], int) or inputs['bound'] == INFINITY:
					return True
			return False
			
		def action_ext_receiving_to_receiving():
			if 'time' in inputs.keys():
				return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : inputs['time'], "bound" : self.state.bound, "elapsed_time" : self.state.elapsed_time}
			if 'bound' in inputs.keys():
				return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : self.state.time, "bound" : inputs['bound'], "elapsed_time" : self.state.elapsed_time}
			if 'add' in inputs.keys():
				self.state.elapsed_time += self.elapsed
				if len(self.state.buffer) < self.state.bound:
					self.state.buffer.insert(0, inputs['add'])
				return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount, "time" : self.state.time, "bound" : self.state.bound, "elapsed_time" : self.state.elapsed_time}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return FIFOState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return FIFOState(name="receiving", **action_ext_receiving_to_receiving())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, generate_value=None):
		self.name = name
		self.period = period
		self.generate_value = generate_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.generate_value = " + str(self.generate_value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, parameters={"name" : "atomic_generator"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=parameters["generate_period"], generate_value=parameters["generate_value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.generate_value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"generate_value" : self.state.generate_value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, parameters={"name" : "Generator"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(parameters={"name" : "generator", "generate_value" : parameters['generate_value'], "generate_period" : parameters['generate_period']}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		return imm[0]

