from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class FIFO_unbounded_untimed_Test(CoupledDEVS):
	def __init__(self, parameters={"name" : "FIFO_unbounded_untimed_Test"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen_value"] = self.addSubModel(Generator(parameters={"name" : "gen_value", "generate_value" : parameters["gen_value_generate_value"], "generate_period" : parameters["gen_value_generate_period"]}))
		self.submodels["gen_get"] = self.addSubModel(Generator(parameters={"name" : "gen_get", "generate_value" : parameters["gen_get_generate_value"], "generate_period" : parameters["gen_get_generate_period"]}))
		self.submodels["fifo"] = self.addSubModel(FIFO_unbounded_untimed(parameters={"name" : "fifo"}))
		self.connectPorts(self.submodels["gen_value"].my_ports["generated"], self.submodels["fifo"].my_ports["add"])
		self.connectPorts(self.submodels["gen_get"].my_ports["generated"], self.submodels["fifo"].my_ports["get"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen_get":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen_value":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "fifo":
					return val
		return imm[0]

class atomic_generatorState:
	def __init__(self, name="", period=None, generate_value=None):
		self.name = name
		self.period = period
		self.generate_value = generate_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.generate_value = " + str(self.generate_value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, parameters={"name" : "atomic_generator"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=parameters["generate_period"], generate_value=parameters["generate_value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.generate_value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"generate_value" : self.state.generate_value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, parameters={"name" : "Generator"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(parameters={"name" : "generator", "generate_value" : parameters['generate_value'], "generate_period" : parameters['generate_period']}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		return imm[0]

class FIFO_unbounded_untimedState:
	def __init__(self, name="", pop_amount=None, buffer=None):
		self.name = name
		self.pop_amount = pop_amount
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.pop_amount = " + str(self.pop_amount) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class FIFO_unbounded_untimed(AtomicDEVS):
	def __init__(self, parameters={"name" : "FIFO_unbounded_untimed"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = FIFO_unbounded_untimedState(name="receiving", pop_amount=1, buffer=[])

		self.my_ports = {"add": self.addInPort("add"), "get": self.addInPort("get"), "output": self.addOutPort("output")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				pops = self.state.pop_amount
				if pops < 1:
					print("requested to pop negative or 0  number of values, will return 1")
					pops = 1
				result = []
				if pops > len(self.state.buffer):
					pops = len(self.state.buffer)
				for x in range(0, pops):
					result.insert(0, self.state.buffer.pop())
				return {"output" : result}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return FIFO_unbounded_untimedState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'get' in inputs.keys():
				return isinstance(inputs['get'], int)
			else:
				return False
			
		def action_ext_receiving_to_sending():
			return {"buffer" : self.state.buffer, "pop_amount" : inputs['get']}
			
		def cond_ext_receiving_to_receiving():
			return 'add' in inputs.keys()
			
		def action_ext_receiving_to_receiving():
			self.state.buffer.insert(0, inputs['add'])
			return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return FIFO_unbounded_untimedState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return FIFO_unbounded_untimedState(name="receiving", **action_ext_receiving_to_receiving())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


