from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Absolute_Test(CoupledDEVS):
	def __init__(self, parameters={"name" : "Absolute_Test"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["generator"] = self.addSubModel(Generator(parameters={"name" : "generator", "generate_value" : parameters["generator_generate_value"], "generate_period" : parameters["generator_generate_period"]}))
		self.submodels["absolute"] = self.addSubModel(Absolute(parameters={"name" : "absolute"}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.submodels["absolute"].my_ports["value"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "Generator":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "Absolute":
					return val
		return imm[0]

class AbsoluteState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Absolute(AtomicDEVS):
	def __init__(self, parameters={"name" : "Absolute"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = AbsoluteState(name="receiving", buffer=0.0)

		self.my_ports = {"value": self.addInPort("value"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "received_absolute":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "received_absolute":
				return {"result" : self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_received_absolute_to_receiving():
			return True
			
		def action_int_received_absolute_to_receiving():
			return {"buffer" : self.state.buffer}
			
		if self.state.name == "received_absolute" and cond_int_received_absolute_to_receiving():
			return AbsoluteState(name="receiving", **action_int_received_absolute_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_received_absolute():
			return isinstance(inputs['value'], (int, long, float))
			
		def action_ext_receiving_to_received_absolute():
			if inputs['value'] < 0:
				return {"buffer" : inputs['value'] * -1}
			else:
				return {"buffer" : inputs['value']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_received_absolute():
			return AbsoluteState(name="received_absolute", **action_ext_receiving_to_received_absolute())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, generate_value=None):
		self.name = name
		self.period = period
		self.generate_value = generate_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.generate_value = " + str(self.generate_value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, parameters={"name" : "atomic_generator"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=parameters["generate_period"], generate_value=parameters["generate_value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.generate_value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"generate_value" : self.state.generate_value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, parameters={"name" : "Generator"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(parameters={"name" : "generator", "generate_value" : parameters['generate_value'], "generate_period" : parameters['generate_period']}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "atomic_generator":
					return val
		return imm[0]

