from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Gain_Test(CoupledDEVS):
	def __init__(self, parameters={"name" : "Gain_Test"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["generator"] = self.addSubModel(Generator(parameters={"name" : "generator", "generate_value" : parameters["generator_generate_value"], "generate_period" : parameters["generator_generate_period"]}))
		self.submodels["gain"] = self.addSubModel(Gain(parameters={"name" : "gain", "gain_factor" : parameters["gain_gain_factor"]}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.submodels["gain"].my_ports["input_factor"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, Generator):
				return val
		for i, val in enumerate(imm):
			if isinstance(val, Gain):
				return val
		return imm[0]

class atomic_generatorState:
	def __init__(self, name="", period=None, generate_value=None):
		self.name = name
		self.period = period
		self.generate_value = generate_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.generate_value = " + str(self.generate_value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, parameters={"name" : "atomic_generator"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=parameters["generate_period"], generate_value=parameters["generate_value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.generate_value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"generate_value" : self.state.generate_value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, parameters={"name" : "Generator"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(parameters={"name" : "generator", "generate_value" : parameters['generate_value'], "generate_period" : parameters['generate_period']}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, atomic_generator):
				return val
		return imm[0]

class GainState:
	def __init__(self, name="", factor=None, buffer=None):
		self.name = name
		self.factor = factor
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.factor = " + str(self.factor) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Gain(AtomicDEVS):
	def __init__(self, parameters={"name" : "Gain"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = GainState(name="receive", factor=parameters['gain_factor'], buffer=0.0)

		self.my_ports = {"input_factor": self.addInPort("input_factor"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "receive":
			return INFINITY
			
		if self.state.name == "do_gain":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receive":
				return {}
				
			if self.state.name == "do_gain":
				return {"result" : self.state.buffer * self.state.factor}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_do_gain_to_receive():
			return True
			
		def action_int_do_gain_to_receive():
			return {"buffer" : 0.0, "factor" : self.state.factor}
			
		if self.state.name == "do_gain" and cond_int_do_gain_to_receive():
			return GainState(name="receive", **action_int_do_gain_to_receive())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receive_to_do_gain():
			return isinstance(inputs['input_factor'], (int, long, float))
			
		def action_ext_receive_to_do_gain():
			return {"buffer" : inputs['input_factor'], "factor" : self.state.factor}
			
		if self.state.name == "receive" and cond_ext_receive_to_do_gain():
			return GainState(name="do_gain", **action_ext_receive_to_do_gain())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


