from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class generate_random_test(CoupledDEVS):
	def __init__(self, parameters={"name" : "generate_random_test"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generate_Random(parameters={"name" : "gen"}))
		self.submodels["abs"] = self.addSubModel(Absolute(parameters={"name" : "abs"}))
		self.submodels["sink"] = self.addSubModel(FIFO_unbounded_untimed(parameters={"name" : "sink"}))

		def transfer_gen_generated_to_abs_value(event):
			return int(event * 10)

		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["abs"].my_ports["value"], transfer_gen_generated_to_abs_value)
		self.connectPorts(self.submodels["abs"].my_ports["result"], self.submodels["sink"].my_ports["add"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "abs":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "sink":
					return val
		return imm[0]

class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, parameters={"name" : "Generate_Random"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=1)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class AbsoluteState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Absolute(AtomicDEVS):
	def __init__(self, parameters={"name" : "Absolute"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = AbsoluteState(name="receiving", buffer=0.0)

		self.my_ports = {"value": self.addInPort("value"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "received_absolute":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "received_absolute":
				return {"result" : self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_received_absolute_to_receiving():
			return True
			
		def action_int_received_absolute_to_receiving():
			return {"buffer" : self.state.buffer}
			
		if self.state.name == "received_absolute" and cond_int_received_absolute_to_receiving():
			return AbsoluteState(name="receiving", **action_int_received_absolute_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_received_absolute():
			return isinstance(inputs['value'], (int, long, float))
			
		def action_ext_receiving_to_received_absolute():
			if inputs['value'] < 0:
				return {"buffer" : inputs['value'] * -1}
			else:
				return {"buffer" : inputs['value']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_received_absolute():
			return AbsoluteState(name="received_absolute", **action_ext_receiving_to_received_absolute())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class FIFO_unbounded_untimedState:
	def __init__(self, name="", pop_amount=None, buffer=None):
		self.name = name
		self.pop_amount = pop_amount
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.pop_amount = " + str(self.pop_amount) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class FIFO_unbounded_untimed(AtomicDEVS):
	def __init__(self, parameters={"name" : "FIFO_unbounded_untimed"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = FIFO_unbounded_untimedState(name="receiving", pop_amount=1, buffer=[])

		self.my_ports = {"add": self.addInPort("add"), "get": self.addInPort("get"), "output": self.addOutPort("output")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				pops = self.state.pop_amount
				if pops < 1:
					print("requested to pop negative or 0  number of values, will return 1")
					pops = 1
				result = []
				if pops > len(self.state.buffer):
					pops = len(self.state.buffer)
				for x in range(0, pops):
					result.insert(0, self.state.buffer.pop())
				return {"output" : result}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return FIFO_unbounded_untimedState(name="receiving", **action_int_sending_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'get' in inputs.keys():
				return isinstance(inputs['get'], int)
			else:
				return False
			
		def action_ext_receiving_to_sending():
			return {"buffer" : self.state.buffer, "pop_amount" : inputs['get']}
			
		def cond_ext_receiving_to_receiving():
			return 'add' in inputs.keys()
			
		def action_ext_receiving_to_receiving():
			self.state.buffer.insert(0, inputs['add'])
			return {"buffer" : self.state.buffer, "pop_amount" : self.state.pop_amount}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return FIFO_unbounded_untimedState(name="sending", **action_ext_receiving_to_sending())


		if self.state.name == "receiving" and cond_ext_receiving_to_receiving():
			return FIFO_unbounded_untimedState(name="receiving", **action_ext_receiving_to_receiving())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


