from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Power_Test(CoupledDEVS):
	def __init__(self, parameters={"name" : "Power_Test"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["value"] = self.addSubModel(Generator(parameters={"name" : "value", "generate_value" : parameters["value_generate_value"], "generate_period" : parameters["value_generate_period"]}))
		self.submodels["factor"] = self.addSubModel(Generator(parameters={"name" : "factor", "generate_value" : parameters["factor_generate_value"], "generate_period" : parameters["factor_generate_period"]}))
		self.submodels["power"] = self.addSubModel(Power(parameters={"name" : "power"}))
		self.connectPorts(self.submodels["value"].my_ports["generated"], self.submodels["power"].my_ports["input_value"])
		self.connectPorts(self.submodels["factor"].my_ports["generated"], self.submodels["power"].my_ports["input_factor"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, Generator):
				return val
		for i, val in enumerate(imm):
			if isinstance(val, Power):
				return val
		return imm[0]

class atomic_generatorState:
	def __init__(self, name="", period=None, generate_value=None):
		self.name = name
		self.period = period
		self.generate_value = generate_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.generate_value = " + str(self.generate_value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, parameters={"name" : "atomic_generator"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=parameters["generate_period"], generate_value=parameters["generate_value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.generate_value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"generate_value" : self.state.generate_value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, parameters={"name" : "Generator"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(parameters={"name" : "generator", "generate_value" : parameters['generate_value'], "generate_period" : parameters['generate_period']}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, atomic_generator):
				return val
		return imm[0]

class PowerState:
	def __init__(self, name="", buffer_factor=None, buffer_value=None):
		self.name = name
		self.buffer_factor = buffer_factor
		self.buffer_value = buffer_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer_factor = " + str(self.buffer_factor) + "\n"
		s += "state.buffer_value = " + str(self.buffer_value) + "\n"
		return s

class Power(AtomicDEVS):
	def __init__(self, parameters={"name" : "Power"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = PowerState(name="zero_received", buffer_factor=1, buffer_value=1)

		self.my_ports = {"input_value": self.addInPort("input_value"), "input_factor": self.addInPort("input_factor"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "zero_received":
			return INFINITY
			
		if self.state.name == "two_received":
			return 0
			
		if self.state.name == "one_received":
			return INFINITY
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "zero_received":
				return {}
				
			if self.state.name == "two_received":
				return {"result" : self.state.buffer_value ** self.state.buffer_factor}
				
			if self.state.name == "one_received":
				return {}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_two_received_to_zero_received():
			return True
			
		def action_int_two_received_to_zero_received():
			return {"buffer_value" : 1, "buffer_factor" : 1}
			
		if self.state.name == "two_received" and cond_int_two_received_to_zero_received():
			return PowerState(name="zero_received", **action_int_two_received_to_zero_received())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_zero_received_to_one_received():
			if 'input_value' in inputs.keys() and 'input_factor' in inputs.keys():
				return isinstance(inputs['input_value'], (int, long, float)) and isinstance(inputs['input_factor'], (int, long, float))
			if 'input_value' in inputs.keys():
				return isinstance(inputs['input_value'], (int, long, float))
			if 'input_factor' in inputs.keys():
				return isinstance(inputs['input_factor'], (int, long, float))
			
		def action_ext_zero_received_to_one_received():
			if 'input_value' in inputs.keys() and 'input_factor' in inputs.keys():
				if isinstance(inputs['input_value'], (int, long, float)) and isinstance(inputs['input_factor'], (int, long, float)):
					return {"buffer_value" : inputs['input_value'], "buffer_factor" : inputs['input_factor']}
			if 'input_value' in inputs.keys():
				if isinstance(inputs['input_value'], (int, long, float)):
					return {"buffer_value" : inputs['input_value'], "buffer_factor" : self.state.buffer_factor}
			if 'input_factor' in inputs.keys():
				if isinstance(inputs['input_factor'], (int, long, float)):
					return {"buffer_value" : self.state.buffer_value, "buffer_factor" : inputs['input_factor']}
			
		def cond_ext_one_received_to_two_received():
			if 'input_value' in inputs.keys() and 'input_factor' in inputs.keys():
				return isinstance(inputs['input_value'], (int, long, float)) and isinstance(inputs['input_factor'], (int, long, float))
			if 'input_value' in inputs.keys():
				return isinstance(inputs['input_value'], (int, long, float))
			if 'input_factor' in inputs.keys():
				return isinstance(inputs['input_factor'], (int, long, float))
			
		def action_ext_one_received_to_two_received():
			if 'input_value' in inputs.keys() and 'input_factor' in inputs.keys():
				if isinstance(inputs['input_value'], (int, long, float)) and isinstance(inputs['input_factor'], (int, long, float)):
					return {"buffer_value" : inputs['input_value'], "buffer_factor" : inputs['input_factor']}
			if 'input_value' in inputs.keys():
				if isinstance(inputs['input_value'], (int, long, float)):
					return {"buffer_value" : inputs['input_value'], "buffer_factor" : self.state.buffer_factor}
			if 'input_factor' in inputs.keys():
				if isinstance(inputs['input_factor'], (int, long, float)):
					return {"buffer_value" : self.state.buffer_value, "buffer_factor" : inputs['input_factor']}
			
		if self.state.name == "zero_received" and cond_ext_zero_received_to_one_received():
			return PowerState(name="one_received", **action_ext_zero_received_to_one_received())


		if self.state.name == "one_received" and cond_ext_one_received_to_two_received():
			return PowerState(name="two_received", **action_ext_one_received_to_two_received())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


