from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class round_test(CoupledDEVS):
	def __init__(self, name = "round_test", round_decimals = 2):
		CoupledDEVS.__init__(self, name)

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generate_Random(name = "gen"))
		self.submodels["neg"] = self.addSubModel(Negative(name = "neg"))
		self.submodels["round"] = self.addSubModel(Round(name = "round", decimals = round_decimals))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["neg"].my_ports["incoming"])
		self.connectPorts(self.submodels["neg"].my_ports["outgoing"], self.submodels["round"].my_ports["incoming"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "gen":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "neg":
					return val
		for i, val in enumerate(imm):
			if isinstance(val, AtomicDEVS) or isinstance(val, CoupledDEVS):
				if val.getModelName() == "round":
					return val
		return imm[0]

class NegativeState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Negative(AtomicDEVS):
	def __init__(self, name = "Negative", buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = NegativeState(name="receiving", buffer=buffer)

		self.my_ports = {"incoming": self.addInPort("incoming"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				return {'outgoing' : self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'buffer' : self.state.buffer}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return NegativeState(name="receiving", **action_int_sending_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'incoming' in inputs.keys() and isinstance(inputs['incoming'], (int, float, long)):
				return True
			return False
			
		def action_ext_receiving_to_sending():
			return {'buffer' : - inputs['incoming']}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return NegativeState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class RoundState:
	def __init__(self, name="", decimals=None, buffer=None):
		self.name = name
		self.decimals = decimals
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.decimals = " + str(self.decimals) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class Round(AtomicDEVS):
	def __init__(self, name = "Round", decimals=0, buffer=None):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = RoundState(name="receiving", decimals=decimals, buffer=buffer)

		self.my_ports = {"incoming": self.addInPort("incoming"), "outgoing": self.addOutPort("outgoing")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "sending":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "sending":
				return {'outgoing' : round(self.state.buffer, self.state.decimals)}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_sending_to_receiving():
			return True
			
		def action_int_sending_to_receiving():
			return {'buffer' : self.state.buffer, 'decimals' : self.state.decimals}
			
		if self.state.name == "sending" and cond_int_sending_to_receiving():
			return RoundState(name="receiving", **action_int_sending_to_receiving())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_sending():
			if 'incoming' in inputs.keys() and isinstance(inputs['incoming'], (int, float, long)):
				return True
			return False
			
		def action_ext_receiving_to_sending():
			return {'buffer' : inputs['incoming'], 'decimals' : self.state.decimals}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_sending():
			return RoundState(name="sending", **action_ext_receiving_to_sending())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Generate_RandomState:
	def __init__(self, name="", period=None):
		self.name = name
		self.period = period


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		return s

class Generate_Random(AtomicDEVS):
	def __init__(self, name = "Generate_Random", period=1):
		AtomicDEVS.__init__(self, name)

		self.my_ports = {}

		self.state = Generate_RandomState(name="generating", period=period)

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				import random
				return {"generated" : random.random()}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return Generate_RandomState(name="generating", **action_int_generating_to_generating())


		else:
			return AtomicDEVS.intTransition(self, my_inputs)

	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

