from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class Sqrt_Test(CoupledDEVS):
	def __init__(self, parameters={"name" : "Sqrt_Test"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["gen"] = self.addSubModel(Generator(parameters={"name" : "gen", "generate_value" : parameters["gen_generate_value"], "generate_period" : parameters["gen_generate_period"]}))
		self.submodels["root"] = self.addSubModel(SQRT(parameters={"name" : "root"}))
		self.connectPorts(self.submodels["gen"].my_ports["generated"], self.submodels["root"].my_ports["input_value"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, Generator):
				return val
		for i, val in enumerate(imm):
			if isinstance(val, SQRT):
				return val
		return imm[0]

class SQRTState:
	def __init__(self, name="", buffer=None):
		self.name = name
		self.buffer = buffer


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer = " + str(self.buffer) + "\n"
		return s

class SQRT(AtomicDEVS):
	def __init__(self, parameters={"name" : "SQRT"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = SQRTState(name="receiving", buffer=1)

		self.my_ports = {"input_value": self.addInPort("input_value"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "receiving":
			return INFINITY
			
		if self.state.name == "calculating_sqrt":
			return 0
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "receiving":
				return {}
				
			if self.state.name == "calculating_sqrt":
				return {"result" : self.state.buffer}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_calculating_sqrt_to_receiving():
			return True
			
		def action_int_calculating_sqrt_to_receiving():
			return {"buffer" : 1}
			
		if self.state.name == "calculating_sqrt" and cond_int_calculating_sqrt_to_receiving():
			return SQRTState(name="receiving", **action_int_calculating_sqrt_to_receiving())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_receiving_to_calculating_sqrt():
			return isinstance(inputs['input_value'], (int, long, float))
			
		def action_ext_receiving_to_calculating_sqrt():
			if inputs['input_value'] < 0:
				print("SQRT cannot take the square root of a negative number, return value = 0")
				return {"buffer" : 0}
			else:
				from math import sqrt
				return {"buffer" : sqrt(inputs['input_value'])}
			
		if self.state.name == "receiving" and cond_ext_receiving_to_calculating_sqrt():
			return SQRTState(name="calculating_sqrt", **action_ext_receiving_to_calculating_sqrt())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class atomic_generatorState:
	def __init__(self, name="", period=None, generate_value=None):
		self.name = name
		self.period = period
		self.generate_value = generate_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.generate_value = " + str(self.generate_value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, parameters={"name" : "atomic_generator"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=parameters["generate_period"], generate_value=parameters["generate_value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.generate_value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"generate_value" : self.state.generate_value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, parameters={"name" : "Generator"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(parameters={"name" : "generator", "generate_value" : parameters['generate_value'], "generate_period" : parameters['generate_period']}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, atomic_generator):
				return val
		return imm[0]

