from pypdevs.DEVS import *
from pypdevs.infinity import INFINITY

class SumTestRoot(CoupledDEVS):
	def __init__(self, parameters={"name" : "SumTestRoot"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {}

		self.submodels["sum"] = self.addSubModel(Sum(parameters={"name" : "sum"}))
		self.submodels["gen1"] = self.addSubModel(Generator(parameters={"name" : "gen1", "generate_value" : parameters["gen1_generate_value"], "generate_period" : parameters["gen1_generate_period"]}))
		self.submodels["gen2"] = self.addSubModel(Generator(parameters={"name" : "gen2", "generate_value" : parameters["gen2_generate_value"], "generate_period" : parameters["gen2_generate_period"]}))
		self.connectPorts(self.submodels["gen1"].my_ports["generated"], self.submodels["sum"].my_ports["adder_one"])
		self.connectPorts(self.submodels["gen2"].my_ports["generated"], self.submodels["sum"].my_ports["adder_two"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, Sum):
				return val
		for i, val in enumerate(imm):
			if isinstance(val, Generator):
				return val
		return imm[0]

class AdderState:
	def __init__(self, name="", buffer_two=None, buffer_one=None):
		self.name = name
		self.buffer_two = buffer_two
		self.buffer_one = buffer_one


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.buffer_two = " + str(self.buffer_two) + "\n"
		s += "state.buffer_one = " + str(self.buffer_one) + "\n"
		return s

class Adder(AtomicDEVS):
	def __init__(self, parameters={"name" : "Adder"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = AdderState(name="zero_received", buffer_two=0.0, buffer_one=0.0)

		self.my_ports = {"adder_one": self.addInPort("adder_one"), "adder_two": self.addInPort("adder_two"), "result": self.addOutPort("result")}

	def timeAdvance(self):
		if self.state.name == "zero_received":
			return INFINITY
			
		if self.state.name == "two_received":
			return 0
			
		if self.state.name == "one_received":
			return INFINITY
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "zero_received":
				return {}
				
			if self.state.name == "two_received":
				return {"result" : self.state.buffer_one + self.state.buffer_two}
				
			if self.state.name == "one_received":
				return {}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_two_received_to_zero_received():
			return True
			
		def action_int_two_received_to_zero_received():
			return {"buffer_one": 0.0, "buffer_two": 0.0}
			
		if self.state.name == "two_received" and cond_int_two_received_to_zero_received():
			return AdderState(name="zero_received", **action_int_two_received_to_zero_received())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		def cond_ext_zero_received_to_one_received():
			if 'adder_one' in inputs.keys() and 'adder_two' in inputs.keys():
				return isinstance(inputs['adder_one'], (int, long, float, complex)) and isinstance(inputs['adder_two'], (int, long, float, complex))
			if 'adder_one' in inputs.keys():
				return isinstance(inputs['adder_one'], (int, long, float, complex))
			if 'adder_two' in inputs.keys():
				return isinstance(inputs['adder_two'], (int, long, float, complex))
			
		def action_ext_zero_received_to_one_received():
			if 'adder_one' in inputs.keys() and 'adder_two' in inputs.keys():
				if isinstance(inputs['adder_one'], (int, long, float, complex)) and isinstance(inputs['adder_two'], (int, long, float, complex)):
					return {"buffer_one" : inputs['adder_one'], "buffer_two" : inputs['adder_two']}
			if 'adder_one' in inputs.keys():
				if isinstance(inputs['adder_one'], (int, long, float, complex)):
					return {"buffer_one" : inputs['adder_one'], "buffer_two" : self.state.buffer_two}
			if 'adder_two' in inputs.keys():
				if isinstance(inputs['adder_two'], (int, long, float, complex)):
					return {"buffer_one" : self.state.buffer_one, "buffer_two" : inputs['adder_two']}
			
		def cond_ext_one_received_to_two_received():
			if 'adder_one' in inputs.keys() and 'adder_two' in inputs.keys():
				return isinstance(inputs['adder_one'], (int, long, float, complex)) and isinstance(inputs['adder_two'], (int, long, float, complex))
			if 'adder_one' in inputs.keys():
				return isinstance(inputs['adder_one'], (int, long, float, complex))
			if 'adder_two' in inputs.keys():
				return isinstance(inputs['adder_two'], (int, long, float, complex))
			
		def action_ext_one_received_to_two_received():
			if 'adder_one' in inputs.keys() and 'adder_two' in inputs.keys():
				if isinstance(inputs['adder_one'], (int, long, float, complex)) and isinstance(inputs['adder_two'], (int, long, float, complex)):
					return {"buffer_one" : inputs['adder_one'], "buffer_two" : inputs['adder_two']}
			if 'adder_one' in inputs.keys():
				if isinstance(inputs['adder_one'], (int, long, float, complex)):
					return {"buffer_one" : inputs['adder_one'], "buffer_two" : self.state.buffer_two}
			if 'adder_two' in inputs.keys():
				if isinstance(inputs['adder_two'], (int, long, float, complex)):
					return {"buffer_one" : self.state.buffer_one, "buffer_two" : inputs['adder_two']}
			
		if self.state.name == "zero_received" and cond_ext_zero_received_to_one_received():
			return AdderState(name="one_received", **action_ext_zero_received_to_one_received())


		if self.state.name == "one_received" and cond_ext_one_received_to_two_received():
			return AdderState(name="two_received", **action_ext_one_received_to_two_received())


		else:
			return AtomicDEVS.extTransition(self, my_inputs)


class Sum(CoupledDEVS):
	def __init__(self, parameters={"name" : "Sum"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"adder_one": self.addInPort("adder_one"), "adder_two": self.addInPort("adder_two"), "result": self.addOutPort("result")}

		self.submodels["adder"] = self.addSubModel(Adder(parameters={"name" : "adder"}))
		self.connectPorts(self.my_ports["adder_one"], self.submodels["adder"].my_ports["adder_one"])
		self.connectPorts(self.my_ports["adder_two"], self.submodels["adder"].my_ports["adder_two"])
		self.connectPorts(self.submodels["adder"].my_ports["result"], self.my_ports["result"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, Adder):
				return val
		return imm[0]

class atomic_generatorState:
	def __init__(self, name="", period=None, generate_value=None):
		self.name = name
		self.period = period
		self.generate_value = generate_value


	def __str__(self):
		s = ""
		s += "state.name = " + str(self.name) + "\n"
		s += "state.period = " + str(self.period) + "\n"
		s += "state.generate_value = " + str(self.generate_value) + "\n"
		return s

class atomic_generator(AtomicDEVS):
	def __init__(self, parameters={"name" : "atomic_generator"}):
		AtomicDEVS.__init__(self, parameters["name"])

		self.my_ports = {}

		self.state = atomic_generatorState(name="generating", period=parameters["generate_period"], generate_value=parameters["generate_value"])

		self.my_ports = {"generated": self.addOutPort("generated")}

	def timeAdvance(self):
		if self.state.name == "generating":
			return self.state.period
			
	def outputFnc(self):
		def subfunc(self):
			if self.state.name == "generating":
				return {'generated' : self.state.generate_value}
				
		return {self.my_ports[k]: v for k, v in subfunc(self).iteritems()}
	def intTransition(self):
		def cond_int_generating_to_generating():
			return True
			
		def action_int_generating_to_generating():
			return {"generate_value" : self.state.generate_value, "period" : self.state.period}
			
		if self.state.name == "generating" and cond_int_generating_to_generating():
			return atomic_generatorState(name="generating", **action_int_generating_to_generating())

		else:
			return AtomicDEVS.intTransition(self)
	def extTransition(self, my_inputs):
		inputs = {k.getPortName(): v for k, v in my_inputs.iteritems()}
		return AtomicDEVS.extTransition(self, my_inputs)

class Generator(CoupledDEVS):
	def __init__(self, parameters={"name" : "Generator"}):
		CoupledDEVS.__init__(self, parameters["name"])

		self.my_ports = {}
		self.submodels = {}

		self.my_ports = {"generated": self.addOutPort("generated")}

		self.submodels["generator"] = self.addSubModel(atomic_generator(parameters={"name" : "generator", "generate_value" : parameters['generate_value'], "generate_period" : parameters['generate_period']}))
		self.connectPorts(self.submodels["generator"].my_ports["generated"], self.my_ports["generated"])

	def select(self, imm):
		for i, val in enumerate(imm):
			if isinstance(val, atomic_generator):
				return val
		return imm[0]

