/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package mdeckers.thesis.uantwerp.gui.editor;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import mdeckers.thesis.uantwerp.gui.graph.AtomicDEVSGraph;
import mdeckers.thesis.uantwerp.gui.main.Log;
import mdeckers.thesis.uantwerp.model.AtomicDEVS;
import mdeckers.thesis.uantwerp.model.Settings;

/**
 *
 * @author Michael
 */
public class AtomicDEVSPanel extends javax.swing.JPanel implements AbstractComponentView {

    AtomicDEVS adevs;
    Settings settings;
    
    /**
     * Creates new form MessagePanel
     */
    public AtomicDEVSPanel(AtomicDEVS adevs, Settings settings) {
        initComponents();
        this.adevs = adevs;
        this.settings = settings;
        
        reset();
        setupListeners();
        setupHighlight();
        
        if (adevs.isLibary() != 0){
            disableAll();
        }
        
    }
    
    private void disableAll(){
        this.saveButton.setEnabled(false);
        this.discardChanges.setEnabled(false);
        this.nameField.setEnabled(false);
        this.parameterNameField.setEnabled(false);
        this.parameterValueField.setEnabled(false);
        this.addParameterButton.setEnabled(false);
        this.parametersField.setEnabled(false);
        this.portsPane.setEnabled(false);
        this.inputRadioButton.setEnabled(false);
        this.outputRadioButton.setEnabled(false);
        this.newPortButton.setEnabled(false);
        this.newPortNameField.setEnabled(false);
        this.statesPane.setEnabled(false);
        this.isStartStateButton.setEnabled(false);
        this.newStateNameField.setEnabled(false);
        this.newStateButton.setEnabled(false);
        this.jCheckBox2.setEnabled(false);
        this.intFromState.setEnabled(false);
        this.intToState.setEnabled(false);
        this.intConditionPane.setEnabled(false);
        this.intActionPane.setEnabled(false);
        this.jCheckBox1.setEnabled(false);
        this.extConditionPane.setEnabled(false);
        this.extActionPane.setEnabled(false);
        this.taField.setEnabled(false);
        this.ofField.setEnabled(false);
    }
    
    public void setupHighlight(){
        
        class Executer {
            public void work(JTextPane parent) {
                parent.setHighlighter(new UnderlineHighlighter(null));
                WordSearcher searcher = new WordSearcher(parent, Color.MAGENTA);
                for (String needle : findParameters(parent.getText())){
                    searcher.search(needle);
                }

                searcher = new WordSearcher(parent, Color.CYAN);
                for (String needle : findPorts(parent.getText())){
                    searcher.search(needle);
                }
                
                searcher = new WordSearcher(parent, Color.GREEN);
                searcher.search("parameters");
            }
            
            private ArrayList<String> findParameters (String text){
                ArrayList<String> states = new ArrayList();
                for (String s : adevs.getParameters()){
                    states.add(s.substring(0, s.indexOf("=")).trim());
                }
                return states;
            }

            private ArrayList<String> findPorts (String text){
                ArrayList<String> states = new ArrayList();
                for (String s : adevs.getPorts()){
                    states.add(s.substring(s.indexOf(":")+1).trim());
                }
                return states;
            }
            
        }
        
        DocumentListener highlight = new DocumentListener() {

            private void warn(DocumentEvent e){
                (new Executer()).work((JTextPane) e.getDocument().getProperty("parent"));
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn(e);
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn(e);
            }

            @Override
            public void changedUpdate(DocumentEvent evt) {
            }
        };
        
        this.parametersField.getDocument().putProperty("parent", parametersField);
        this.parametersField.getDocument().addDocumentListener(highlight);
        (new Executer()).work(parametersField);

        this.portsPane.getDocument().putProperty("parent", portsPane);
        this.portsPane.getDocument().addDocumentListener(highlight);
        (new Executer()).work(portsPane);
        
        this.taField.getDocument().putProperty("parent", taField);
        this.taField.getDocument().addDocumentListener(highlight);
        (new Executer()).work(taField);
        
        this.ofField.getDocument().putProperty("parent", ofField);
        this.ofField.getDocument().addDocumentListener(highlight);
        (new Executer()).work(ofField);
        
        this.intActionPane.getDocument().putProperty("parent", intActionPane);
        this.intActionPane.getDocument().addDocumentListener(highlight);
        (new Executer()).work(intActionPane);
        
        this.intConditionPane.getDocument().putProperty("parent", intConditionPane);
        this.intConditionPane.getDocument().addDocumentListener(highlight);
        (new Executer()).work(intConditionPane);
        
        this.extActionPane.getDocument().putProperty("parent", extActionPane);
        this.extActionPane.getDocument().addDocumentListener(highlight);
        (new Executer()).work(extActionPane);
        
        this.extConditionPane.getDocument().putProperty("parent", extConditionPane);
        this.extConditionPane.getDocument().addDocumentListener(highlight);
        (new Executer()).work(extConditionPane);

        
        
    }
    
    HashMap<String, String> taMap;
    HashMap<String, String> ofMap;
    
    class ExternalTransition {
        private final String from, to;
        private final String condition, action;
        private boolean enabled;
        
        public ExternalTransition(String from, String to, String condition, String action){
            this.from = from;
            this.to = to;
            this.condition = condition;
            this.action = action;
            this.enabled = true;
        }
        
        public String getFrom(){return from;}
        public String getTo(){return to;}
        public String getCondition(){return condition;}
        public String getAction(){return action;}
        public void setEnabled(boolean enabled) {this.enabled = enabled;}
        public boolean isEnabled(){return this.enabled;}
        
    }
    
    ArrayList<ExternalTransition> ETs;
    
    class InternalTransition {
        private final String from, to;
        private final String condition, action;
        private boolean enabled;
        
        public InternalTransition(String from, String to, String condition, String action){
            this.from = from;
            this.to = to;
            this.condition = condition;
            this.action = action;
            this.enabled = true;
        }
        
        public String getFrom(){return from;}
        public String getTo(){return to;}
        public String getCondition(){return condition;}
        public String getAction(){return action;}
        public void setEnabled(boolean enabled) {this.enabled = enabled;}
        public boolean isEnabled(){return this.enabled;}
        
    }
    
    ArrayList<InternalTransition> ITs;
    
    
    
    public void reset(){
        taMap = new HashMap();
        ofMap = new HashMap();
        if (adevs != null){
            parametersField.setText("");
            portsPane.setText("");
            statesPane.setText("");
            taField.setText("");
            ofField.setText("");
            intConditionPane.setText("");
            intActionPane.setText("");
            extConditionPane.setText("");
            extActionPane.setText("");
            
            nameField.setText(adevs.getName());
            
            for (String s : adevs.getPorts()){
                portsPane.setText(portsPane.getText() + s + "\n");
            }
            
            for (String s : adevs.getParameters()){
                parametersField.setText(parametersField.getText() + s + "\n");
            }
            
            for (String s : adevs.getStates()){
                statesPane.setText(statesPane.getText() + s + "\n");
            }
            
            if (statesPane.getText().contains("starting state")){
                isStartStateButton.setSelected(false);
                isStartStateButton.setEnabled(false);
            } else {
                isStartStateButton.setSelected(true);
                isStartStateButton.setEnabled(true);
            }
            
            
            
            
            ArrayList<String> states = new ArrayList();
            for (String s : adevs.getStates()){
                if (s.contains(" : starting state")){
                    states.add(s.substring(0, s.indexOf(" : starting state")));
                } else {
                    states.add(s);
                }
            }
            String[] list = new String[states.size()];
            list = states.toArray(list);
            taSelectState.setModel(new javax.swing.DefaultComboBoxModel<>(list));
            intFromState.setModel(new javax.swing.DefaultComboBoxModel<>(list));
            intToState.setModel(new javax.swing.DefaultComboBoxModel<>(list));
            extFromState.setModel(new javax.swing.DefaultComboBoxModel<>(list));
            extToState.setModel(new javax.swing.DefaultComboBoxModel<>(list));
            ofSelectState.setModel(new javax.swing.DefaultComboBoxModel<>(list));
            
            for (ArrayList<String> s : adevs.getTAs()){
                for (int i = 0; i < s.size(); i++){
                    if (i == 0){
                        taMap.put(s.get(i), "");
                    } else {
                        taMap.replace(s.get(0), taMap.get(s.get(0)).concat(s.get(i) + "\n"));
                    }
                }
                if (s.get(0).equals((String) taSelectState.getSelectedItem())){
                    for (int i = 1; i < s.size(); i++){
                        taField.setText(taField.getText() + s.get(i) + "\n");
                    }
                }
            }
            
            for (ArrayList<String> s : adevs.getOFs()){
                for (int i = 0; i < s.size(); i++){
                    if (i == 0){
                        ofMap.put(s.get(i), "");
                    } else {
                        ofMap.replace(s.get(0), ofMap.get(s.get(0)).concat(s.get(i) + "\n"));
                    }
                }
                if (s.get(0).equals((String) ofSelectState.getSelectedItem())){
                    for (int i = 1; i < s.size(); i++){
                        ofField.setText(ofField.getText() + s.get(i) + "\n");
                    }
                }
            }
            
            
            /*if (adevs.getITs() == null || adevs.getITs().isEmpty()){
                jCheckBox2.setSelected(false);
                jCheckBox2ActionPerformed(null);
            } else {
                jCheckBox2.setSelected(true);
            }
            for (ArrayList<String> s : adevs.getITs()){
                System.out.println(s);
                if (s.size() == 1){
                    if (s.get(0).contains("from_state")){
                        this.intFromState.setSelectedItem(s.get(0).substring(s.get(0).indexOf(":")+1).trim());
                    } else if (s.get(0).contains("to_state")){
                        this.intToState.setSelectedItem(s.get(0).substring(s.get(0).indexOf(":")+1).trim());
                    }
                } else {
                    if (s.get(0).equals("condition")){
                        for (int p = 1; p < s.size(); p++){
                            this.intConditionPane.setText(this.intConditionPane.getText() + s.get(p) + "\n");
                        }
                    } else if (s.get(0).equals("action")){
                        for (int p = 1; p < s.size(); p++){
                            this..setText(this.intActionPane.getText() + s.get(p) + "\n");
                        }intActionPane
                    }
                }
            }*/
            
            ITs = new ArrayList();
            if (adevs.getITs() != null){
                String from = "", to = "", condition = "", action = "";
                for (int c = 0; c < adevs.getITs().size(); c++){
                    ArrayList<String> array = adevs.getITs().get(c);
                    if (c % 4 == 0){
                        from = array.get(0).substring(array.get(0).indexOf(":")+2);
                    }
                    if (c % 4 == 1){
                        to = array.get(0).substring(array.get(0).indexOf(":")+2);
                    }
                    if (c % 4 == 2){
                        condition = "";
                        for (int i = 1; i < array.size(); i++){
                            condition += array.get(i) + "\n";
                        }
                    }
                    if (c % 4 == 3){
                        action = "";
                        for (int i = 1; i < array.size(); i++){
                            action += array.get(i) + "\n";
                        }
                        ITs.add(new InternalTransition(from, to, condition, action));
                        //HashMap<String, InternalTransition> map = new HashMap();
                        //map.put(to, new InternalTransition(from, to, condition, action));
                        //ITs.put(from, map);
                    }
                }
            }
            
            if (intFromState.getModel().getSize() > 0)
                intFromState.setSelectedIndex(0);
            if (intToState.getModel().getSize() > 0)
                intToState.setSelectedIndex(0);
            
            InternalTransition loadInt = null;
            try {
                for (InternalTransition et : ITs){
                    if (et.getFrom().equals((String) intFromState.getSelectedItem())){
                        if (et.getTo().equals((String) intToState.getSelectedItem())){
                            loadInt = et;
                            break;
                        }
                    }
                }
            } catch (NullPointerException e) {
                loadInt = null;
            }
            if (loadInt == null){
                jCheckBox2.setSelected(false);
                jCheckBox2ActionPerformed(null);
                intConditionPane.setText("");
                intActionPane.setText("");
            } else {
                jCheckBox2.setSelected(loadInt.isEnabled());
                jCheckBox2ActionPerformed(null);
                intConditionPane.setText(loadInt.getCondition());
                intActionPane.setText(loadInt.getAction());
            }
            
            ETs = new ArrayList();
            if (adevs.getETs() != null){
                String from = "", to = "", condition = "", action = "";
                for (int c = 0; c < adevs.getETs().size(); c++){
                    ArrayList<String> array = adevs.getETs().get(c);
                    if (c % 4 == 0){
                        from = array.get(0).substring(array.get(0).indexOf(":")+2);
                    }
                    if (c % 4 == 1){
                        to = array.get(0).substring(array.get(0).indexOf(":")+2);
                    }
                    if (c % 4 == 2){
                        condition = "";
                        for (int i = 1; i < array.size(); i++){
                            condition += array.get(i) + "\n";
                        }
                    }
                    if (c % 4 == 3){
                        action = "";
                        for (int i = 1; i < array.size(); i++){
                            action += array.get(i) + "\n";
                        }
                        ETs.add(new ExternalTransition(from, to, condition, action));
                        //HashMap<String, ExternalTransition> map = new HashMap();
                        //map.put(to, new ExternalTransition(from, to, condition, action));
                        //ETs.put(from, map);
                    }
                }
            }
            
            if (extFromState.getModel().getSize() > 0)
                extFromState.setSelectedIndex(0);
            if (extToState.getModel().getSize() > 0)
                extToState.setSelectedIndex(0);
            
            ExternalTransition load = null;
            try {
                for (ExternalTransition et : ETs){
                    if (et.getFrom().equals((String) extFromState.getSelectedItem())){
                        if (et.getTo().equals((String) extToState.getSelectedItem())){
                            load = et;
                            break;
                        }
                    }
                }
            } catch (NullPointerException e) {
                load = null;
            }
            if (load == null){
                jCheckBox1.setSelected(false);
                jCheckBox1ActionPerformed(null);
                extConditionPane.setText("");
                extActionPane.setText("");
            } else {
                jCheckBox1.setSelected(load.isEnabled());
                jCheckBox1ActionPerformed(null);
                extConditionPane.setText(load.getCondition());
                extActionPane.setText(load.getAction());
            }
            
            
            
            
            /*if (adevs.getETs() == null || adevs.getETs().isEmpty()){
                jCheckBox1.setSelected(false);
                jCheckBox1ActionPerformed(null);
            } else {
                jCheckBox1.setSelected(true);
            }
            for (ArrayList<String> s : adevs.getETs()){
                if (s.size() == 1){
                    if (s.get(0).contains("from_state")){
                        this.extFromState.setSelectedItem(s.get(0).substring(s.get(0).indexOf(":")+1).trim());
                    } else if (s.get(0).contains("to_state")){
                        this.extToState.setSelectedItem(s.get(0).substring(s.get(0).indexOf(":")+1).trim());
                    }
                } else {
                    if (s.get(0).equals("condition")){
                        for (int p = 1; p < s.size(); p++){
                            this.extConditionPane.setText(this.extConditionPane.getText() + s.get(p) + "\n");
                        }
                    } else if (s.get(0).equals("action")){
                        for (int p = 1; p < s.size(); p++){
                            this.extActionPane.setText(this.extActionPane.getText() + s.get(p) + "\n");
                        }
                    }
                }
            }*/
            
            if (statesPane.getText().replace("\n", "").trim().isEmpty()){
                jTabbedPane2.setEnabledAt(1, false);
                jTabbedPane2.setEnabledAt(2, false);
                taField.setEnabled(false);
                ofField.setEnabled(false);
            } else {
                jTabbedPane2.setEnabledAt(1, true);
                jTabbedPane2.setEnabledAt(2, true);
                taField.setEnabled(true);
                ofField.setEnabled(true);
            }
        }
    }
    
    
    
    @Override
    public void save(){
        transferSelectionChangedET("from", (String) extFromState.getSelectedItem());
        transferSelectionChangedIT("from", (String) intFromState.getSelectedItem());
        adevs.setName(nameField.getText());
        
        ArrayList<String> params = new ArrayList();
        String[] lines = statesPane.getText().concat("\n").split("\\n");
        for (String line : lines){
            if (line.trim().length() > 0){
                params.add(line.trim());
            }
        }
        adevs.setStates(params);
        
        params = new ArrayList();
        lines = portsPane.getText().concat("\n").split("\\n");
        for (String line : lines){
            if (line.trim().length() > 0){
                params.add(line.trim());
            }
        }
        adevs.setPorts(params);
        
        params = new ArrayList();
        lines = this.parametersField.getText().concat("\n").split("\\n");
        for (String line : lines){
            if (line.trim().length() > 0){
                params.add(line.trim());
            }
        }
        adevs.setParameters(params);
        
        ArrayList<ArrayList<String>> params2 = new ArrayList();
        Iterator it = taMap.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry<String, String> pair = (Map.Entry)it.next();
            params = new ArrayList();
            params.add(pair.getKey());
            lines = pair.getValue().concat("\n").split("\\n");
            for (String line : lines){
                if (line.length() > 0){
                    params.add(line);
                }
            }
            params2.add(params);
            //it.remove();
        }
        adevs.setTAs(params2);
        
        params2 = new ArrayList();
        it = ofMap.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry<String, String> pair = (Map.Entry)it.next();
            params = new ArrayList();
            params.add(pair.getKey());
            lines = pair.getValue().concat("\n").split("\\n");
            for (String line : lines){
                if (line.length() > 0){
                    params.add(line);
                }
            }
            params2.add(params);
            //it.remove();
        }
        adevs.setOFs(params2);
        
        
        
        /*params2 = new ArrayList();
        params = new ArrayList();
        params.add("from_state : " + this.intFromState.getSelectedItem());
        params2.add(params);
        params = new ArrayList();
        params.add("to_state : " + this.intToState.getSelectedItem());
        params2.add(params);
        params = new ArrayList();
        params.add("condition");
        lines = this.intConditionPane.getText().concat("\n").split("\\n");
        for (String line : lines){
            //if (line.trim().length() > 0){
                params.add(line);
            //}
        }
        params2.add(params);
        params = new ArrayList();
        params.add("action");
        lines = this.intActionPane.getText().concat("\n").split("\\n");
        for (String line : lines){
            //if (line.trim().length() > 0){
                params.add(line);
            //}
        }
        params2.add(params);
        if (jCheckBox2.isSelected())
            adevs.setITs(params2);
        else
            adevs.setITs(new ArrayList<ArrayList<String>>());*/
        
        params2 = new ArrayList();
        
        //it = ETs.entrySet().iterator();
        for (InternalTransition et : ITs){
            if (et != null && et.isEnabled()){
                params = new ArrayList();
                params.add("from_state : " + et.from);
                params2.add(params);
                params = new ArrayList();
                params.add("to_state : " + et.to);
                params2.add(params);
                params = new ArrayList();
                params.add("condition");
                lines = et.getCondition().concat("\n").split("\\n");
                for (String line : lines){
                    //if (line.trim().length() > 0){
                        params.add(line);
                    //}
                }
                params2.add(params);
                params = new ArrayList();
                params.add("action");
                lines = et.getAction().concat("\n").split("\\n");
                for (String line : lines){
                    //if (line.trim().length() > 0){
                        params.add(line);
                    //}
                }
                params2.add(params);
                params = new ArrayList();
            }
        }
        adevs.setITs(params2);
        
        /*params2 = new ArrayList();
        params = new ArrayList();
        params.add("from_state : " + this.extFromState.getSelectedItem());
        params2.add(params);
        params = new ArrayList();
        params.add("to_state : " + this.extToState.getSelectedItem());
        params2.add(params);
        params = new ArrayList();
        params.add("condition");
        lines = this.extConditionPane.getText().concat("\n").split("\\n");
        for (String line : lines){
            if (line.trim().length() > 0){
                params.add(line.trim());
            }
        }
        params2.add(params);
        params = new ArrayList();
        params.add("action");
        lines = this.extActionPane.getText().concat("\n").split("\\n");
        for (String line : lines){
            if (line.trim().length() > 0){
                params.add(line.trim());
            }
        }
        params2.add(params);
        if (jCheckBox1.isSelected())
            adevs.setETs(params2);
        else
            adevs.setETs(new ArrayList<ArrayList<String>>());
        */
        
        params2 = new ArrayList();
        
        //it = ETs.entrySet().iterator();
        for (ExternalTransition et : ETs){
            if (et != null && et.isEnabled()){
                params = new ArrayList();
                params.add("from_state : " + et.from);
                params2.add(params);
                params = new ArrayList();
                params.add("to_state : " + et.to);
                params2.add(params);
                params = new ArrayList();
                params.add("condition");
                lines = et.getCondition().concat("\n").split("\\n");
                for (String line : lines){
                    //if (line.trim().length() > 0){
                        params.add(line);
                    //}
                }
                params2.add(params);
                params = new ArrayList();
                params.add("action");
                lines = et.getAction().concat("\n").split("\\n");
                for (String line : lines){
                    //if (line.trim().length() > 0){
                        params.add(line);
                    //}
                }
                params2.add(params);
                params = new ArrayList();
            }
        }
        adevs.setETs(params2);
        
        adevs.save();
        Log.message("Save complete.");
    }
    
    
    private void setupListeners(){
        
        taSelectState.addActionListener (new ActionListener () {
            @Override
            public void actionPerformed(ActionEvent e) {
                
                //taField.setText(taMap.get(taSelectState.getSelectedItem()));
            }
        });
        
        taSelectState.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.DESELECTED){
                    taMap.put((String) e.getItem(), taField.getText());
                } else if (e.getStateChange() == ItemEvent.SELECTED){
                    taField.setText(taMap.get(taSelectState.getSelectedItem()));
                }
            }

        });
        
        
        
        ofSelectState.addActionListener (new ActionListener () {
            @Override
            public void actionPerformed(ActionEvent e) {
                //ofField.setText(ofMap.get(ofSelectState.getSelectedItem()));
            }
        });
        
        ofSelectState.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.DESELECTED){
                    ofMap.put((String) e.getItem(), ofField.getText());
                } else if (e.getStateChange() == ItemEvent.SELECTED){
                    ofField.setText(ofMap.get(ofSelectState.getSelectedItem()));
                }
            }

        });
        
        
        
        taField.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
               taMap.put((String) taSelectState.getSelectedItem(), taField.getText());
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        
        ofField.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
               ofMap.put((String) ofSelectState.getSelectedItem(), ofField.getText());
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        

        statesPane.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
                if (statesPane.getText().contains("starting state")){
                    isStartStateButton.setSelected(false);
                    isStartStateButton.setEnabled(false);
                } else {
                    isStartStateButton.setSelected(true);
                    isStartStateButton.setEnabled(true);
                }
               
               if (statesPane.getText().replace("\n", "").trim().isEmpty()){
                    jTabbedPane2.setEnabledAt(1, false);
                    jTabbedPane2.setEnabledAt(2, false);
                    taField.setEnabled(false);
                    ofField.setEnabled(false);
                } else {
                    jTabbedPane2.setEnabledAt(1, true);
                    jTabbedPane2.setEnabledAt(2, true);
                    taField.setEnabled(true);
                    ofField.setEnabled(true);
                }
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        
        nameField.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
               String text = nameField.getText();
               if (text.trim().length() == 0){
                   saveButton.setEnabled(false);
               } else {
                   saveButton.setEnabled(true);
               }
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        
        newStateNameField.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
               String text = newStateNameField.getText();
               if (text.trim().length() == 0){
                   newStateButton.setEnabled(false);
               } else {
                   newStateButton.setEnabled(true);
               }
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        
        discardChanges.addActionListener(new ActionListener() { 
            @Override
            public void actionPerformed(ActionEvent e) {
                JDialog.setDefaultLookAndFeelDecorated(true);
                int response = JOptionPane.showConfirmDialog(null, "Are you sure you want to discard all changes?", "Confirm",
                    JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                if (response == JOptionPane.YES_OPTION) {
                  reset();
                } 
            }
          } );
        
        saveButton.addActionListener(new ActionListener() { 
            @Override
            public void actionPerformed(ActionEvent e) {
                save();
            }
          } );
        
        
        parameterNameField.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
               parameterNameFieldActionPerformed(null);
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        
        parameterValueField.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
               parameterNameFieldActionPerformed(null);
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        
        newPortNameField.getDocument().addDocumentListener(new DocumentListener() {

            public void warn() {
               String text = newPortNameField.getText();
               if (text.trim().length() == 0){
                   newPortButton.setEnabled(false);
               } else {
                   newPortButton.setEnabled(true);
               }
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                warn();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                warn();
            }
          });
        
        newPortButton.addActionListener(new ActionListener() { 
            @Override
            public void actionPerformed(ActionEvent e) {
                if (outputRadioButton.isSelected())
                    portsPane.setText(portsPane.getText() + "OUT : " + newPortNameField.getText() + "\n");
                else 
                    portsPane.setText(portsPane.getText() + "IN : " + newPortNameField.getText() + "\n");
                newPortNameField.setText("");
                newPortButton.setEnabled(false);
            }
          } );
        
        extFromState.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.DESELECTED){
                    transferSelectionChangedET("from", (String) e.getItem());
                }
            }

        });
        
        extToState.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.DESELECTED){
                    transferSelectionChangedET("to", (String) e.getItem());
                }
            }
            
        });
        
        intFromState.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.DESELECTED){
                    transferSelectionChangedIT("from", (String) e.getItem());
                }
            }

        });
        
        intToState.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.DESELECTED){
                    transferSelectionChangedIT("to", (String) e.getItem());
                }
            }
            
        });
    }
    
    private void transferSelectionChangedET(String origin, String string) {
        ExternalTransition et;
        String from, to;
        if (origin.equals("from")){
            from = string;
            to = (String) extToState.getSelectedItem();
        } else {
            to = string;
            from = (String) extFromState.getSelectedItem();
        }
        et = new ExternalTransition(from, to, this.extConditionPane.getText(), this.extActionPane.getText());
        if (!this.jCheckBox1.isSelected()){
            et.setEnabled(false);
        }
        
        int index = -1;
        try {
            for (ExternalTransition ets : ETs){
                if (ets.getFrom().equals(et.getFrom())){
                    if (ets.getTo().equals(et.getTo())){
                        index = ETs.indexOf(ets);
                    }
                }
            }
            if (index != -1){
                ETs.remove(index);
            }
            ETs.add(et);
        } catch (NullPointerException e){}
        
    }
    
    private void transferSelectionChangedIT(String origin, String string) {
        InternalTransition et;
        String from, to;
        if (origin.equals("from")){
            from = string;
            to = (String) intToState.getSelectedItem();
        } else {
            to = string;
            from = (String) intFromState.getSelectedItem();
        }
        et = new InternalTransition(from, to, this.intConditionPane.getText(), this.intActionPane.getText());
        if (!this.jCheckBox2.isSelected()){
            et.setEnabled(false);
        }
        
        int index = -1;
        try {
            for (InternalTransition ets : ITs){
                if (ets.getFrom().equals(et.getFrom())){
                    if (ets.getTo().equals(et.getTo())){
                        index = ITs.indexOf(ets);
                    }
                }
            }
            if (index != -1){
                ITs.remove(index);
            }
            ITs.add(et);
        } catch (NullPointerException e){}
        
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        jSeparator1 = new javax.swing.JSeparator();
        saveButton = new javax.swing.JButton();
        discardChanges = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        jTabbedPane2 = new javax.swing.JTabbedPane();
        jPanel4 = new javax.swing.JPanel();
        nameField = new javax.swing.JTextField();
        jPanel2 = new javax.swing.JPanel();
        addParameterButton = new javax.swing.JButton();
        parameterNameField = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        parameterValueField = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        parametersField = new javax.swing.JTextPane();
        jPanel3 = new javax.swing.JPanel();
        inputRadioButton = new javax.swing.JRadioButton();
        outputRadioButton = new javax.swing.JRadioButton();
        jLabel3 = new javax.swing.JLabel();
        newPortNameField = new javax.swing.JTextField();
        newPortButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        portsPane = new javax.swing.JTextPane();
        jPanel6 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        newStateNameField = new javax.swing.JTextField();
        newStateButton = new javax.swing.JButton();
        isStartStateButton = new javax.swing.JCheckBox();
        jScrollPane3 = new javax.swing.JScrollPane();
        statesPane = new javax.swing.JTextPane();
        jPanel7 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        intFromState = new javax.swing.JComboBox<>();
        jLabel6 = new javax.swing.JLabel();
        intToState = new javax.swing.JComboBox<>();
        jCheckBox2 = new javax.swing.JCheckBox();
        jScrollPane10 = new javax.swing.JScrollPane();
        intConditionPane = new javax.swing.JTextPane();
        jScrollPane4 = new javax.swing.JScrollPane();
        intActionPane = new javax.swing.JTextPane();
        jPanel8 = new javax.swing.JPanel();
        jPanel9 = new javax.swing.JPanel();
        jLabel7 = new javax.swing.JLabel();
        extFromState = new javax.swing.JComboBox<>();
        extToState = new javax.swing.JComboBox<>();
        jLabel8 = new javax.swing.JLabel();
        jCheckBox1 = new javax.swing.JCheckBox();
        jScrollPane5 = new javax.swing.JScrollPane();
        extConditionPane = new javax.swing.JTextPane();
        jScrollPane6 = new javax.swing.JScrollPane();
        extActionPane = new javax.swing.JTextPane();
        jPanel11 = new javax.swing.JPanel();
        jLabel10 = new javax.swing.JLabel();
        taSelectState = new javax.swing.JComboBox<>();
        jScrollPane7 = new javax.swing.JScrollPane();
        taField = new javax.swing.JTextPane();
        jPanel12 = new javax.swing.JPanel();
        jLabel11 = new javax.swing.JLabel();
        ofSelectState = new javax.swing.JComboBox<>();
        jScrollPane9 = new javax.swing.JScrollPane();
        ofField = new javax.swing.JTextPane();

        setPreferredSize(new java.awt.Dimension(993, 637));

        saveButton.setText("Save changes");
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        discardChanges.setText("Discard changes");
        discardChanges.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                discardChangesActionPerformed(evt);
            }
        });

        jButton1.setText("Show Visualization");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        nameField.setFont(new java.awt.Font("Courier New", 0, 11)); // NOI18N
        nameField.setBorder(javax.swing.BorderFactory.createTitledBorder("Name"));
        nameField.setCaretColor(null);
        nameField.setMinimumSize(new java.awt.Dimension(6, 50));
        nameField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nameFieldActionPerformed(evt);
            }
        });

        jPanel2.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        addParameterButton.setText(">");
        addParameterButton.setEnabled(false);
        addParameterButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addParameterButtonActionPerformed(evt);
            }
        });

        parameterNameField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                parameterNameFieldActionPerformed(evt);
            }
        });

        jLabel2.setText("Name:");

        jLabel1.setText("Value:");

        parameterValueField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                parameterValueFieldActionPerformed(evt);
            }
        });

        parametersField.setBorder(javax.swing.BorderFactory.createTitledBorder("State parameters"));
        jScrollPane1.setViewportView(parametersField);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addComponent(parameterValueField, javax.swing.GroupLayout.PREFERRED_SIZE, 129, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel2Layout.createSequentialGroup()
                            .addComponent(jLabel2)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(parameterNameField, javax.swing.GroupLayout.PREFERRED_SIZE, 129, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(addParameterButton, javax.swing.GroupLayout.Alignment.TRAILING))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 220, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel2)
                            .addComponent(parameterNameField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(20, 20, 20)
                        .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel1)
                            .addComponent(parameterValueField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(addParameterButton)
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addComponent(jScrollPane1)
                        .addContainerGap())))
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        buttonGroup1.add(inputRadioButton);
        inputRadioButton.setSelected(true);
        inputRadioButton.setText("Input");
        inputRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                inputRadioButtonActionPerformed(evt);
            }
        });

        buttonGroup1.add(outputRadioButton);
        outputRadioButton.setText("Output");

        jLabel3.setText("Name:");

        newPortButton.setText("^");
        newPortButton.setEnabled(false);
        newPortButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newPortButtonActionPerformed(evt);
            }
        });

        portsPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Ports"));
        jScrollPane2.setViewportView(portsPane);

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane2)
                    .addGroup(jPanel3Layout.createSequentialGroup()
                        .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(jPanel3Layout.createSequentialGroup()
                                .addComponent(jLabel3)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(newPortNameField, javax.swing.GroupLayout.DEFAULT_SIZE, 156, Short.MAX_VALUE))
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel3Layout.createSequentialGroup()
                                .addComponent(inputRadioButton)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(outputRadioButton)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(newPortButton)))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 195, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(outputRadioButton)
                    .addComponent(inputRadioButton))
                .addGap(4, 4, 4)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(newPortNameField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(newPortButton))
                .addContainerGap())
        );

        jPanel6.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        jLabel4.setText("Name:");

        newStateNameField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newStateNameFieldActionPerformed(evt);
            }
        });

        newStateButton.setText("^");
        newStateButton.setEnabled(false);
        newStateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newStateButtonActionPerformed(evt);
            }
        });

        isStartStateButton.setText("Starting state");

        statesPane.setBorder(javax.swing.BorderFactory.createTitledBorder("States"));
        jScrollPane3.setViewportView(statesPane);

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 233, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel6Layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addGap(3, 3, 3)
                        .addComponent(newStateNameField)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(newStateButton))
                    .addGroup(jPanel6Layout.createSequentialGroup()
                        .addComponent(isStartStateButton)
                        .addGap(0, 142, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(isStartStateButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(newStateNameField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(newStateButton))
                .addContainerGap())
        );

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(nameField, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 947, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel4Layout.createSequentialGroup()
                        .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(11, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGap(6, 6, 6)
                .addComponent(nameField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel6, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(210, 210, 210))
        );

        jTabbedPane2.addTab("AtomicDEVS state", jPanel4);
        jPanel4.getAccessibleContext().setAccessibleName("AtomicDEVS state");

        jLabel5.setText("From state:");

        intFromState.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        intFromState.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                intFromStateActionPerformed(evt);
            }
        });

        jLabel6.setText("To state:");

        intToState.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        intToState.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                intToStateActionPerformed(evt);
            }
        });

        jCheckBox2.setSelected(true);
        jCheckBox2.setText("Enable Internal Transition for this combination of states");
        jCheckBox2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox2ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jCheckBox2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 115, Short.MAX_VALUE)
                .addComponent(jLabel5)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(intFromState, javax.swing.GroupLayout.PREFERRED_SIZE, 205, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel6)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(intToState, javax.swing.GroupLayout.PREFERRED_SIZE, 205, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(jLabel5)
                .addComponent(intFromState, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(jLabel6)
                .addComponent(intToState, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(jCheckBox2))
        );

        intConditionPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Condition"));
        jScrollPane10.setViewportView(intConditionPane);

        intActionPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Action"));
        jScrollPane4.setViewportView(intActionPane);

        javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
        jPanel7.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane4)
                    .addComponent(jScrollPane10)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane10, javax.swing.GroupLayout.PREFERRED_SIZE, 233, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane4, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPane2.addTab("Internal Transition", jPanel7);

        jLabel7.setText("From state:");

        extFromState.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        extFromState.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                extFromStateActionPerformed(evt);
            }
        });

        extToState.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        extToState.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                extToStateActionPerformed(evt);
            }
        });

        jLabel8.setText("To state:");

        jCheckBox1.setSelected(true);
        jCheckBox1.setText("Enable External Transition for this combination of states");
        jCheckBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel9Layout = new javax.swing.GroupLayout(jPanel9);
        jPanel9.setLayout(jPanel9Layout);
        jPanel9Layout.setHorizontalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel9Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jCheckBox1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 113, Short.MAX_VALUE)
                .addComponent(jLabel7)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(extFromState, javax.swing.GroupLayout.PREFERRED_SIZE, 205, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel8)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(extToState, javax.swing.GroupLayout.PREFERRED_SIZE, 205, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        jPanel9Layout.setVerticalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel9Layout.createSequentialGroup()
                .addGap(0, 0, Short.MAX_VALUE)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(extFromState, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8)
                    .addComponent(extToState, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jCheckBox1)))
        );

        extConditionPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Condition"));
        jScrollPane5.setViewportView(extConditionPane);

        extActionPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Action"));
        jScrollPane6.setViewportView(extActionPane);

        javax.swing.GroupLayout jPanel8Layout = new javax.swing.GroupLayout(jPanel8);
        jPanel8.setLayout(jPanel8Layout);
        jPanel8Layout.setHorizontalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane6)
                    .addComponent(jScrollPane5)
                    .addComponent(jPanel9, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel8Layout.setVerticalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane5, javax.swing.GroupLayout.PREFERRED_SIZE, 235, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane6, javax.swing.GroupLayout.DEFAULT_SIZE, 230, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPane2.addTab("External Transition", jPanel8);

        jLabel10.setText("Select state: ");

        taSelectState.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        taSelectState.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                taSelectStateActionPerformed(evt);
            }
        });

        taField.setBorder(javax.swing.BorderFactory.createTitledBorder("Time Advance"));
        jScrollPane7.setViewportView(taField);

        javax.swing.GroupLayout jPanel11Layout = new javax.swing.GroupLayout(jPanel11);
        jPanel11.setLayout(jPanel11Layout);
        jPanel11Layout.setHorizontalGroup(
            jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel11Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane7)
                    .addGroup(jPanel11Layout.createSequentialGroup()
                        .addComponent(jLabel10)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(taSelectState, javax.swing.GroupLayout.PREFERRED_SIZE, 180, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 700, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel11Layout.setVerticalGroup(
            jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel11Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel10)
                    .addComponent(taSelectState, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane7, javax.swing.GroupLayout.DEFAULT_SIZE, 494, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPane2.addTab("Time Advance", jPanel11);

        jLabel11.setText("Select state: ");

        ofSelectState.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        ofField.setBorder(javax.swing.BorderFactory.createTitledBorder("Output Function"));
        jScrollPane9.setViewportView(ofField);

        javax.swing.GroupLayout jPanel12Layout = new javax.swing.GroupLayout(jPanel12);
        jPanel12.setLayout(jPanel12Layout);
        jPanel12Layout.setHorizontalGroup(
            jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel12Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane9)
                    .addGroup(jPanel12Layout.createSequentialGroup()
                        .addComponent(jLabel11)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ofSelectState, javax.swing.GroupLayout.PREFERRED_SIZE, 180, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 700, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel12Layout.setVerticalGroup(
            jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel12Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel11)
                    .addComponent(ofSelectState, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane9, javax.swing.GroupLayout.DEFAULT_SIZE, 494, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPane2.addTab("Output Function", jPanel12);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSeparator1)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jTabbedPane2)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(saveButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(discardChanges)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(jButton1)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(saveButton)
                    .addComponent(discardChanges)
                    .addComponent(jButton1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jTabbedPane2)
                .addContainerGap())
        );

        jTabbedPane2.getAccessibleContext().setAccessibleName("AtomicDEVS");
    }// </editor-fold>//GEN-END:initComponents

    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_saveButtonActionPerformed

    private void newStateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newStateButtonActionPerformed
        if (this.isStartStateButton.isSelected())
            statesPane.setText(statesPane.getText() + newStateNameField.getText().trim() + " : starting state\n");
        else 
            statesPane.setText(statesPane.getText() + newStateNameField.getText().trim() + "\n");
        newStateNameField.setText("");
        newStateButton.setEnabled(false);
        
        if (statesPane.getText().replace("\n", "").trim().isEmpty()){
            jTabbedPane2.setEnabledAt(1, false);
            jTabbedPane2.setEnabledAt(2, false);
            taField.setEnabled(false);
            ofField.setEnabled(false);
        } else {
            jTabbedPane2.setEnabledAt(1, true);
            jTabbedPane2.setEnabledAt(2, true);
            taField.setEnabled(true);
            ofField.setEnabled(true);
        }
        
        this.parameterNameFieldActionPerformed(evt);
    }//GEN-LAST:event_newStateButtonActionPerformed

    private void newPortButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newPortButtonActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_newPortButtonActionPerformed

    private void inputRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_inputRadioButtonActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_inputRadioButtonActionPerformed

    private void parameterNameFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_parameterNameFieldActionPerformed
        if (parameterNameField.getText().trim().length() != 0 && parameterValueField.getText().length() != 0){
            addParameterButton.setEnabled(true);
        } else {
            addParameterButton.setEnabled(false);
        }
    }//GEN-LAST:event_parameterNameFieldActionPerformed

    private void nameFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nameFieldActionPerformed

    }//GEN-LAST:event_nameFieldActionPerformed

    private void parameterValueFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_parameterValueFieldActionPerformed
        this.parameterNameFieldActionPerformed(evt);
    }//GEN-LAST:event_parameterValueFieldActionPerformed

    private void addParameterButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addParameterButtonActionPerformed
        parametersField.setText(parametersField.getText() + parameterNameField.getText().trim() + " = " + parameterValueField.getText() + "\n");
        parameterNameField.setText("");
        addParameterButton.setEnabled(false);
        parameterValueField.setText("");
    }//GEN-LAST:event_addParameterButtonActionPerformed

    private void jCheckBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox1ActionPerformed
        boolean set = jCheckBox1.isSelected();
        this.extActionPane.setEnabled(set);
        this.extConditionPane.setEnabled(set);
    }//GEN-LAST:event_jCheckBox1ActionPerformed

    private void jCheckBox2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox2ActionPerformed
        boolean set = jCheckBox2.isSelected();
        this.intActionPane.setEnabled(set);
        this.intConditionPane.setEnabled(set);
    }//GEN-LAST:event_jCheckBox2ActionPerformed

    private void newStateNameFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newStateNameFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_newStateNameFieldActionPerformed

    private void extFromStateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_extFromStateActionPerformed
        ExternalTransition load = null;
        try {
            for (ExternalTransition et : ETs){
                    if (et.getFrom().equals((String) extFromState.getSelectedItem())){
                        if (et.getTo().equals((String) extToState.getSelectedItem())){
                            load = et;
                            break;
                        }
                    }
                }
        } catch (NullPointerException e) {
            load = null;
        }
        if (load == null){
            jCheckBox1.setSelected(false);
            jCheckBox1ActionPerformed(null);
            extConditionPane.setText("");
            extActionPane.setText("");
        } else {
            jCheckBox1.setSelected(load.isEnabled());
            jCheckBox1ActionPerformed(null);
            extConditionPane.setText(load.getCondition());
            extActionPane.setText(load.getAction());
        }
    }//GEN-LAST:event_extFromStateActionPerformed

    private void extToStateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_extToStateActionPerformed
        ExternalTransition load = null;
        try {
            for (ExternalTransition et : ETs){
                    if (et.getFrom().equals((String) extFromState.getSelectedItem())){
                        if (et.getTo().equals((String) extToState.getSelectedItem())){
                            load = et;
                            break;
                        }
                    }
                }
        } catch (NullPointerException e) {
            load = null;
        }
        if (load == null){
            jCheckBox1.setSelected(false);
            jCheckBox1ActionPerformed(null);
            extConditionPane.setText("");
            extActionPane.setText("");
        } else {
            jCheckBox1.setSelected(load.isEnabled());
            jCheckBox1ActionPerformed(null);
            extConditionPane.setText(load.getCondition());
            extActionPane.setText(load.getAction());
        }
    }//GEN-LAST:event_extToStateActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        JOptionPane.showMessageDialog(null,
            "Unsaved changes will not be shown.", "Notice", JOptionPane.PLAIN_MESSAGE);
        AtomicDEVSGraph adg = new AtomicDEVSGraph(adevs);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void taSelectStateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_taSelectStateActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_taSelectStateActionPerformed

    private void intFromStateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_intFromStateActionPerformed
        InternalTransition load = null;
        try {
            for (InternalTransition et : ITs){
                    if (et.getFrom().equals((String) intFromState.getSelectedItem())){
                        if (et.getTo().equals((String) intToState.getSelectedItem())){
                            load = et;
                            break;
                        }
                    }
                }
        } catch (NullPointerException e) {
            load = null;
        }
        if (load == null){
            jCheckBox2.setSelected(false);
            jCheckBox2ActionPerformed(null);
            intConditionPane.setText("");
            intActionPane.setText("");
        } else {
            jCheckBox2.setSelected(load.isEnabled());
            jCheckBox2ActionPerformed(null);
            intConditionPane.setText(load.getCondition());
            intActionPane.setText(load.getAction());
        }
    }//GEN-LAST:event_intFromStateActionPerformed

    private void intToStateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_intToStateActionPerformed
        InternalTransition load = null;
        try {
            for (InternalTransition et : ITs){
                    if (et.getFrom().equals((String) intFromState.getSelectedItem())){
                        if (et.getTo().equals((String) intToState.getSelectedItem())){
                            load = et;
                            break;
                        }
                    }
                }
        } catch (NullPointerException e) {
            load = null;
        }
        if (load == null){
            jCheckBox2.setSelected(false);
            jCheckBox2ActionPerformed(null);
            intConditionPane.setText("");
            intActionPane.setText("");
        } else {
            jCheckBox2.setSelected(load.isEnabled());
            jCheckBox2ActionPerformed(null);
            intConditionPane.setText(load.getCondition());
            intActionPane.setText(load.getAction());
        }
    }//GEN-LAST:event_intToStateActionPerformed

    private void discardChangesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_discardChangesActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_discardChangesActionPerformed

    

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addParameterButton;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JButton discardChanges;
    private javax.swing.JTextPane extActionPane;
    private javax.swing.JTextPane extConditionPane;
    private javax.swing.JComboBox<String> extFromState;
    private javax.swing.JComboBox<String> extToState;
    private javax.swing.JRadioButton inputRadioButton;
    private javax.swing.JTextPane intActionPane;
    private javax.swing.JTextPane intConditionPane;
    private javax.swing.JComboBox<String> intFromState;
    private javax.swing.JComboBox<String> intToState;
    private javax.swing.JCheckBox isStartStateButton;
    private javax.swing.JButton jButton1;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JCheckBox jCheckBox2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane10;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JScrollPane jScrollPane6;
    private javax.swing.JScrollPane jScrollPane7;
    private javax.swing.JScrollPane jScrollPane9;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JTabbedPane jTabbedPane2;
    private javax.swing.JTextField nameField;
    private javax.swing.JButton newPortButton;
    private javax.swing.JTextField newPortNameField;
    private javax.swing.JButton newStateButton;
    private javax.swing.JTextField newStateNameField;
    private javax.swing.JTextPane ofField;
    private javax.swing.JComboBox<String> ofSelectState;
    private javax.swing.JRadioButton outputRadioButton;
    private javax.swing.JTextField parameterNameField;
    private javax.swing.JTextField parameterValueField;
    private javax.swing.JTextPane parametersField;
    private javax.swing.JTextPane portsPane;
    private javax.swing.JButton saveButton;
    private javax.swing.JTextPane statesPane;
    private javax.swing.JTextPane taField;
    private javax.swing.JComboBox<String> taSelectState;
    // End of variables declaration//GEN-END:variables
}
